/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 28-02-2005		  Butano/Zoppello      Corretto metodo releaseResources per chiudere 
* 										   i cursori eventualmente contenuti in un 
* 										   CompositeDataResult     										    
**/
/*
 * @author
 */
import java.util.Iterator;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.DataResultInterface;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.dbaccess.sql.result.std.CompositeDataResult;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

public class Utils {
    private Utils() {
        super();
    } // private Utils()

    public static EMFInternalError generateInternalError(
        Exception ex,
        String method) {
        return new EMFInternalError(EMFErrorSeverity.ERROR, method, ex);
    } // public static EMFInternalError generateInternalError(Exception ex, String method)

    public static void releaseResources(
        DataConnection connection,
        SQLCommand command,
        DataResult result) {
        if (result != null) {
            DataResultInterface resultInterface = result.getDataObject();
            if ((resultInterface != null)
                && (resultInterface instanceof ScrollableDataResult)) {
                try {
                    ((ScrollableDataResult) resultInterface).close();
                } // try
                catch (EMFInternalError ie) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.MAJOR,
                        "Utils::releaseResources: ((ScrollableDataResult)resultInterface).close()",
                        (Exception) ie);
                } // catch (EMFInternalError ie)
            } // if ((resultInterface != null) && (resultInterface
            // instanceof ScrollableDataResult))
            if ((resultInterface != null) && (resultInterface instanceof CompositeDataResult)) {
                    List containedDataResults = ((CompositeDataResult) resultInterface).getContainedDataResult();

                    try {
                        DataResultInterface localDr = null;
                        for (Iterator it = containedDataResults.iterator(); it.hasNext();) {
                            localDr = (DataResultInterface) it.next();
                            if (localDr instanceof ScrollableDataResult) {
                                ((ScrollableDataResult) localDr).close();
                            }
                        }
                    } catch (EMFInternalError ie) {
                        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
                                "Utils::releaseResources: ((ScrollableDataResult)resultInterface).close()", (Exception) ie);
                    } // catch (EMFInternalError ie)
                } // if ((resultInterface != null) && (resultInterface
                // instanceof CompositeDataResult))    
        } // if (result != null)
        if (command != null) {
            try {
                command.close();
            } // try
            catch (EMFInternalError ie) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.MAJOR,
                    "Utils::releaseResources: command.close()",
                    (Exception) ie);
            } // catch (EMFInternalError ie)
        } // if (command != null)
        if (connection != null) {
            try {
                connection.close();
            } // try
            catch (EMFInternalError ie) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.MAJOR,
                    "Utils::releaseResources: connection.close()",
                    (Exception) ie);
            } // catch (EMFInternalError ie)
        } // if (connection != null)
    } // public static void releaseResources(DataConnection connection, SQLCommand command, DataResult result)
} // public class Utils
