/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
/* Generated by Together */

package it.eng.spago.dbaccess.factory;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.ConnectionPoolDescriptor;
import it.eng.spago.dbaccess.pool.ConnectionPoolInterface;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Questa classe  responsabile per la creazione di un ConnectionPool dato il suo descrittore
 * questa factory invoca attraverso l'introspezione la connectionPoolFactory class
 * descritta nel file xml quando il pool viene descritto
 * @author Andrea Zoppello 
 * @version 1.0
 */
public class ConnectionPoolFactory {
    /**
    * Il factory method responsabile per la creazione di un pool dato il suo descrittore
    * @param <B>ConnectionPoolDescriptor</B> connectionPoolDescriptor - l'oggetto descrittore del pool da creare
    * @return  un oggetto di tipo <B>ConnectionPoolInterface</B> rappresentante il pool di connessioni appena creato
    */
    public static ConnectionPoolInterface createConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor)
        throws EMFInternalError {
            try {
                ConnectionPoolInterface returnedConnectionPool = null;
                Class connectionPoolFactoryClass = Class.forName(connectionPoolDescriptor.getConnectionPoolFactory());
                Object connectionPoolFactoryObject = connectionPoolFactoryClass.newInstance();
                Class[] parameterTypesOfFactoryMethod = new Class[] {
                    ConnectionPoolDescriptor.class
                };
                Method factoryMethod;
                Object[] argumentsOfFactoryMethod = new Object[] {
                    connectionPoolDescriptor
                };
                factoryMethod = connectionPoolFactoryClass.getMethod("createConnectionPool", parameterTypesOfFactoryMethod);
                returnedConnectionPool = (ConnectionPoolInterface)
                    factoryMethod.invoke(connectionPoolFactoryObject, argumentsOfFactoryMethod);
                return returnedConnectionPool;
            }
            catch (ClassNotFoundException cnfe) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "ConnectionPoolFactory::createConnectionPool:", cnfe);
                throw new EMFInternalError(EMFErrorSeverity.ERROR,
                    "ConnectionPoolFactory :: createConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor) :: CLASS NOT FOUND EXCEPTION " +
                    cnfe.getMessage());
            }
            catch (NoSuchMethodException nsme) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "ConnectionPoolFactory::createConnectionPool:", nsme);
                throw new EMFInternalError(EMFErrorSeverity.ERROR,
                    "ConnectionPoolFactory :: createConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor) :: NOSUCH METHOD EXCEPTION " +
                    nsme.getMessage());
            }
            catch (InvocationTargetException ite) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "ConnectionPoolFactory::createConnectionPool:", ite);
                Throwable e = ite.getTargetException();
                if (e instanceof EMFInternalError)
                    throw((EMFInternalError)e);
                else
                    throw new EMFInternalError(EMFErrorSeverity.ERROR,
                        "ConnectionPoolFactory :: createConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor) :: " +
                        ite.getMessage());
            }
            catch (IllegalAccessException iae) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "ConnectionPoolFactory::createConnectionPool:", iae);
                throw new EMFInternalError(EMFErrorSeverity.ERROR,
                    "ConnectionPoolFactory :: createConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor) :: ILLEGAL ACCESS EXCEPTION " +
                    iae.getMessage());
            }
            catch (InstantiationException ie) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "ConnectionPoolFactory::createConnectionPool:", ie);
                throw new EMFInternalError(EMFErrorSeverity.ERROR,
                    "ConnectionPoolFactory :: createConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor) :: INSTANTIATION EXCEPTION " +
                    ie.getMessage());
            } //end try-catch
    } //public static ConnectionPoolInterface
    // createConnectionPool(ConnectionPoolDescriptor connectionPoolDescriptor)
} //end Class ConnectionPoolDataSourceFactory
