/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.sql.command.legacy;

import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.command.CommandCreator;
import it.eng.spago.dbaccess.sql.command.std.StoredProcedureCommand;

/**
* Questa Classe  una factory concreta di CommandCreator per la creazione di oggetti SQLCommand
* che fanno uso delle sole caratteristiche di JDBC 1.0
*
* @author Andrea Zoppello 
* @version 1.0
*/
public class LegacyCommandCreator implements CommandCreator {
    /**
    * Crea un SQLCommand di select data una DataConnection e la CommandString per driver jdbc 1.0
    * @param DataConnection dataConnection - la connessione al db
    * @param String commandString - La stringa contenente il comando sql
    * @return un oggetto di tipo <B>SQLCommand</B> rapprsentante il comando di select contenuto nella command string
    */
    public SQLCommand createSelectCommand(
        DataConnection dataConnection,
        String commandString,
        boolean scroll) {
        return new LegacySelectCommand(dataConnection, commandString, scroll);
    } // public SQLCommand createSelectCommand(DataConnection dataConnection, String commandString, boolean scroll)

    /**
    * Crea un SQLCommand di insert data una DataConnection e la CommandString per driver jdbc 1.0
    * @param DataConnection dataConnection - la connessione al db
    * @param String commandString - La stringa contenente il comando sql
    * @return un oggetto di tipo <B>SQLCommand</B> rapprsentante il comando di insert contenuto nella command string
    */
    public SQLCommand createInsertCommand(
        DataConnection dataConnection,
        String commandString) {
        return new LegacyInsertCommand(dataConnection, commandString);
    } //public SQLCommand createInsertCommand(DataConnection dataConnection,
    // String commandString)

    /**
    * Crea un SQLCommand di update data una DataConnection e la CommandString  per driver jdbc 1.0
    * @param DataConnection dataConnection - la connessione al db
    * @param String commandString - La stringa contenente il comando sql
    * @return un oggetto di tipo <B>SQLCommand</B> rapprsentante il comando di update contenuto nella command string
    */
    public SQLCommand createUpdateCommand(
        DataConnection dataConnection,
        String commandString) {
        return new LegacyUpdateCommand(dataConnection, commandString);
    } //public SQLCommand createUpdateCommand(DataConnection dataConnection,
    // String commandString)

    /**
    * Crea un SQLCommand di delete data una DataConnection e la CommandString  per driver jdbc 1.0
    * @param DataConnection dataConnection - la connessione al db
    * @param String commandString - La stringa contenente il comando sql
    * @return un oggetto di tipo <B>SQLCommand</B> rapprsentante il comando di delete contenuto nella command string
    */
    public SQLCommand createDeleteCommand(
        DataConnection dataConnection,
        String commandString) {
        return new LegacyDeleteCommand(dataConnection, commandString);
    } //public SQLCommand createDeleteCommand(DataConnection dataConnection,
    // String commandString)

    /**
    * Crea un SQLCommand per l'esecuzione di una stored procedure data una DataConnection e la CommandString
    * @param DataConnection dataConnection - la connessione al db
    * @param <B>String</B>  commandString - La stringa contenente il comando sql
    * @return un oggetto di tipo <B>SQLCommand</B> rapprsentante il comando di delete contenuto nella command string
    */
    public SQLCommand createStoredProcedureCommand(
        DataConnection dataConnection,
        String commandString) {
        return new StoredProcedureCommand(dataConnection, commandString);
    } //public SQLCommand createStoredProcedureCommand(DataConnection
    // dataConnection, String commandString)
} //end Class LegacyCommandCreator
