/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.sql.mappers;

/**
* Questa interfaccia definisce le funzioni di un SQLMapper che  responsabile per la conversione da oggetti a stringa e viceversa per gli oggetti di tipo
* sql.DATE e sql.TIMESTAMP che non sono standard e variano a seconda del vendor
*
* @author Andrea Zoppello 
* @version 1.0
*/
public interface SQLMapper {
    String getStringValue(int sqlType, Object objectValue);

    Object getObjectValue(int sqlType, String stringValue);

    /**
    * Questo metodo converte un valore data di tipo Stringa in un oggetto
    * @param <B>String</B> il valore di una data in una Stringa
    * @return <B>Object</B> L'oggetto rappresentante la data
    */
    Object getObjectValueForDate(String stringDateValue);

    /**
    * Questo metodo converte un valore timestamp di tipo Stringa in un oggetto
    * @param <B>String</B> il valore di un timestamp in una stringa
    * @return <B>Object</B> L'oggetto rappresentante il timeStamp
    */
    Object getObjectValueForTimeStamp(String stringTimeStampValue);

    /**
    * Questo metodo converte un valore timestamp di tipo Object in un oggetto di tipo String
    * @param <B>Object</B> objectTimeStampValue - L'oggetto rappresentante il timeStamp
    * @return <B>String</B> il valore in un oggetto di tipo String
    */
    String getStringValueForTimeStamp(Object objectTimeStampValue);

    /**
    * Questo metodo converte un valore data di tipo Object in un oggetto di tipo String
    * @param <B>Object</B> objectDateValue - L'oggetto rappresentante la data
    * @return <B>String</B> il valore in un oggetto di tipo String
    */
    String getStringValueForDate(Object objectDateValue);
} //end Interface SQLMapper
