/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
/* Generated by Together */

package it.eng.spago.dbaccess.sql.result;

import java.util.List;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;

/**
* Questa classe rappresenta un PucntulaDataResult, cio un DataResult che contiene un DataField
*
* Tipicamente un oggetto di questo tipo viene utilizzato per memorizzare i valori dei paremetri di ritorno o
* di output di una stored procedure che non sono di tipo ResultSet.
*
* @author Andrea Zoppello 
* @version 1.0
*/
public class PunctualDataResult implements DataResultInterface {
    private DataField _punctualDataField = null;
    //private SQLCommand _sqlCommand = null;
    public static final String TAG = "PUNCTUAL_DATA_RESULT";

    /**
    * Crea un oggetto DataResultInterface di tipo PunctualDataResult.
    * @param <B>SQLCommand</B> command - Il comando che ha generato come risultato l'oggetto PunctualDataResult che verr creato da questa funzione
    * @param <B>List</B> inputParameters - La lista dei parametri di input del comando che ha generato come risultato l'oggetto PunctualDataResult che verr creato da questa funzione
    * @param <B>DataField</B> punctualData - Il dataField che costituice il dato puntuale alla base dell'oggetto PunctualDataResult creato da questa funzione
    * @throws <B>EMFInternalError</B> - Se qualche errore si verifica nella creazione dell'oggetto fi tipo PunctualDataResult
    */
    public PunctualDataResult(SQLCommand sqlCommand, List inputParameters, DataField punctualDataField)
        throws EMFInternalError {
            //_sqlCommand = sqlCommand;
            _punctualDataField = punctualDataField;
    } //public PunctualDataResult(SQLCommand sqlCommand, List inputParameters,
    // DataField punctualDataField) throws EMFInternalError

    /**
    * Ritorna l'oggetto DataField contenuto nell'oggetto.
    * @return l'oggetto <B>DataField</B> contenuto nell'oggetto
    */
    public DataField getPunctualDatafield() {
        return _punctualDataField;
    } //public DataField getPunctualDatafield()

    /**
    * Ritorna come valore la costante DataResultInterface.PUNCTUAL_DATA_RESULT.
    * @return DataResultInterface.PUNCTUAL_DATA_RESULT
    */
    public String getDataResultType() {
        return DataResultInterface.PUNCTUAL_DATA_RESULT;
    } //public String getDataResultType()

    /**
    * Ritorna un oggetto <code>Sourcebean</code> rappresentante l'oggetto.
    * @return la rappresentazione dell'oggetto di tipo <B>DataResultInterface</B> come <B>SourceBean</B>
    */
    public SourceBean getSourceBean() throws EMFInternalError {
        try {
            SourceBean sourceBean = new SourceBean(PunctualDataResult.TAG);
            sourceBean.setAttribute("PUNCTUAL_DATA_FIELD", _punctualDataField);
            return sourceBean;
        }
        catch (SourceBeanException sbe) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR,
                "InformationDataResul  :: getSourceBean() ::  " + sbe.getMessage());
        }
    } //public SourceBean getSourceBean()
} //end Class PunctualDataResult
