/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
/* Generated by Together */

package it.eng.spago.dbaccess.sql.result.std;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResultInterface;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
* Questa Classe rappresenta il risultato di un'esecuzione di un comando che abbia pi risultati di output
* Per esempio una stored procedure con un valore di ritorno e due parametri di output
*
* @author Andrea Zoppello 
* @version 1.0
*/
public class CompositeDataResult implements DataResultInterface {
    private List containedDataResult = null;
    public static final String TAG = "COMPOSITE_DATA_RESULT";

    public CompositeDataResult(SQLCommand command, List inputParameters) {
        containedDataResult = new ArrayList();
    }

    /**
     * Permette di aggiungere un oggetto dataResult al contenitore.
     * @param <code>dataResult</code> oggetto che rappresenta un valore di ritorno.
     */
    public void add(DataResultInterface dataResult) {
        containedDataResult.add(dataResult);
    }

    /**
     * Permette di recuperare la lista degli oggetti .
     * @return <code>List</code> la lista degli oggetti.
     */
    public List getContainedDataResult() {
        return containedDataResult;
    }

    /**
     * Ad ogni oggetto della lista viene chiesto di ritornare un'istanza di SourceBean contenente
     * lo stato dell'oggetto;tutte le nuove istanze vengono ritornare incapsulate in un altro oggetto
     * SourceBean.
     * @return <code>SourceBean</code> contenente la lista delle istanze SourceBean .
     * @exception EMFInternalError viene lanciata se non  possibile popolare un'istanza di SourceBean.
     */
    public SourceBean getSourceBean() throws EMFInternalError {
        Monitor monitor =
            MonitorFactory.start(
                "model.data-access.composite-data-result.get-source-bean");
        try {
            SourceBean sourceBean = new SourceBean(CompositeDataResult.TAG);
            Iterator it = containedDataResult.iterator();
            while (it.hasNext()) {
                sourceBean.setAttribute(
                    ((DataResultInterface) it.next()).getSourceBean());
            }
            return sourceBean;
        }
        catch (SourceBeanException sbe) {
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "CompositeDataResult  :: getSourceBean() ::  "
                    + sbe.getMessage());
        }
        finally {
            monitor.stop();
        }
    }
}
