/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.action;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

public class ActionFactory {
    private ActionFactory() {
        super();
    } // private ActionFactory()

    public static ActionIFace getAction(RequestContainer requestContainer) throws ActionException {
        SourceBean serviceRequest = (SourceBean)requestContainer.getServiceRequest();
        String actionName = (String)serviceRequest.getAttribute(Constants.ACTION_NAME);
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "ActionFactory::getAction: actionName [" + actionName + "]");
        ConfigSingleton configure = ConfigSingleton.getInstance();
        SourceBean actionBean = (SourceBean)configure.getFilteredSourceBeanAttribute("ACTIONS.ACTION", "NAME", actionName);
        if (actionBean == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ActionFactory::getAction: nome della action non valido !");
            throw new ActionException("Nome della action [" + actionName + "] non valido");
        } // if (actionBean == null)
        String actionClass = (String)actionBean.getAttribute("CLASS");
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "ActionFactory::getAction: actionClass [" + actionClass + "]");
        SourceBean actionConfig = (SourceBean)actionBean.getAttribute("CONFIG");
        if (actionConfig == null)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "ActionFactory::getAction: config della action non censito");
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "ActionFactory::getAction: actionConfig", actionConfig);
        String actionScope = (String)actionBean.getAttribute("SCOPE");
        if (actionScope == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ActionFactory::getAction: scope della action non censito, default [REQUEST]");
            actionScope = "REQUEST";
        } // if (actionScope == null)
        if ((actionScope == null) || (!actionScope.equalsIgnoreCase("REQUEST") && !actionScope.equalsIgnoreCase("SESSION") &&
            !actionScope.equalsIgnoreCase("APPLICATION"))) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "ActionFactory::getAction: scope della action non valido [" + actionScope + "]");
                actionScope = "REQUEST";
        } // if ((actionScope == null) ||
        // (!actionScope.equalsIgnoreCase("REQUEST") &&
        // !actionScope.equalsIgnoreCase("SESSION") &&
        // !actionScope.equalsIgnoreCase("APPLICATION")))
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "ActionFactory::getAction: actionScope [" + actionScope + "]");
        if (actionScope.equalsIgnoreCase("REQUEST")) {
            ActionIFace action = null;
            try {
                action = (ActionIFace)Class.forName(actionClass).newInstance();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "ActionFactory::getAction: classe della action non valida", ex);
                throw new ActionException("Classe della action [" + actionName + "] non valida");
            } // catch (Exception ex) try
            action.setActionName(actionName);
            ((InitializerIFace)action).init(actionConfig);
            return action;
        } // if (actionScope.equalsIgnoreCase("REQUEST"))
        else if (actionScope.equalsIgnoreCase("SESSION")) {
            SessionContainer sessionContainer = requestContainer.getSessionContainer().getPermanentContainer();
            ActionIFace action = (ActionIFace)sessionContainer.getAttribute("AF_ACTION_" + actionName);
            if (action == null) {
                synchronized(sessionContainer) {
                    action = (ActionIFace)sessionContainer.getAttribute("AF_ACTION_" + actionName);
                    if (action == null) {
                        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                            "ActionFactory::getAction: action non trovata in SESSION !");
                        try {
                            action = (ActionIFace)Class.forName(actionClass).newInstance();
                        } // try
                        catch (Exception ex) {
                            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                                "ActionFactory::getAction: classe della action [" + actionName + "] non valida", ex);
                            throw new ActionException("Classe della action [" + actionName + "] non valida");
                        } // catch (Exception ex) try
                        action.setActionName(actionName);
                        ((InitializerIFace)action).init(actionConfig);
                        sessionContainer.setAttribute("AF_ACTION_" + actionName, action);
                    } // if (action == null)
                } // synchronized (sessionContainer)
            } // if (action == null)
            else
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "ActionFactory::getAction: action trovata in SESSION");
            return action;
        } // if (actionScope.equalsIgnoreCase("SESSION"))
        else {
            ApplicationContainer applicationContainer = ApplicationContainer.getInstance();
            ActionIFace action = (ActionIFace)applicationContainer.getAttribute("AF_ACTION_" + actionName);
            if (action == null) {
                synchronized(applicationContainer) {
                    action = (ActionIFace)applicationContainer.getAttribute("AF_ACTION_" + actionName);
                    if (action == null) {
                        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                            "ActionFactory::getAction: action non trovata in APPLICATION !");
                        try {
                            action = (ActionIFace)Class.forName(actionClass).newInstance();
                        } // try
                        catch (Exception ex) {
                            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                                "ActionFactory::getAction: classe della action [" + actionName + "] non valida", ex);
                            throw new ActionException("Classe della action [" + actionName + "] non valida");
                        } // catch (Exception ex) try
                        action.setActionName(actionName);
                        ((InitializerIFace)action).init(actionConfig);
                        applicationContainer.setAttribute("AF_ACTION_" + actionName, action);
                    } // if (action == null)
                } // synchronized (applicationContainer)
            } // if (action == null)
            else
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "ActionFactory::getAction: action trovata in APPLICATION");
            return action;
        } // if (actionScope.equalsIgnoreCase("SESSION")) else
    } // public static ActionIFace getAction(RequestContainer requestContainer) throws ActionException
} // public class ActionFactory
