/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.coordinator;

import it.eng.spago.dispatching.service.RequestContextIFace;

/**
 * L'interfaccia  <code>DispatcherIFace</code> identifica i servizi che un dispatcher deve erogare.
 * Il dispatcher deve implementare nel metodo <code>acceptsURL(RequestContextIFace requestContext)</code>
 * la logica con la quale stabilisce se  in grado di gestire una richiesta di servizio. Nel metodo
 * <code>getCoordinator(RequestContextIFace requestContext)</code> il dispatcher deve ritornare l'istanza di
 * <code>CoordinatorIFace</code> che gestir la richiesta.
 * @author Luigi Bellio
 * @see CoordinatorIFace
 */
public interface DispatcherIFace {
    /**
     * Ritorna un <code>boolean</code> che avr il seguente significato:
     * <em>true</em> se il dispatcher accetta di gestire la richiesta
     * <em>false</em> se il dispatcher non accetta di gestire la richiesta
     * @return <code>boolean</code> che indica se il dispatcher accetta di gestire la richiesta.
     */
    boolean acceptsURL(RequestContextIFace requestContext);

    String getBusinessType(RequestContextIFace requestContext);

    String getBusinessName(RequestContextIFace requestContext);

    /**
     * Ritorna un <code>CoordinatorIFace</code> che avr il compito di gestire la richiesta.
     * @param requestContext il contesto del servizio
     * @return <code>CoordinatorIFace</code> il gestore del servizio.
     */
    CoordinatorIFace getCoordinator(RequestContextIFace requestContext);
} // public interface DispatcherIFace
