/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.module.detail;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.module.AbstractModule;
import it.eng.spago.dispatching.service.detail.IFaceDetailService;
import it.eng.spago.dispatching.service.detail.impl.DelegatedDetailService;
import it.eng.spago.tracing.TracerSingleton;

/**
 * La classe <code>AbstractDetailModule</code> &egrave; la superclasse di un' implementazione
 * di un dettaglio generalizzato.
 *
 * @author Luigi Bellio
 * @see it.eng.spago.base.SourceBean
 */
public abstract class AbstractDetailModule extends AbstractModule implements IFaceDetailService {
    public AbstractDetailModule() {
        super();
    } // public AbstractDetailModule()

    /**
     * @see it.eng.spago.dispatching.service.ServiceIFace#service(SourceBean, SourceBean)
     */
    public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "AbstractDetailModule::service: invocato");
        DelegatedDetailService.service(this, serviceRequest, serviceResponse);
    } // public void service(SourceBean serviceRequest, SourceBean
    // serviceResponse) throws Exception

    /**
     * @see it.eng.spago.dispatching.service.detail.IFaceDetailService#insert(SourceBean, SourceBean)
     */
    public boolean insert(SourceBean request, SourceBean response) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
            "AbstractDetailModule::insert: metodo non implementato !");
        return false;
    } // public boolean insert(SourceBean request, SourceBean response)

    /**
     * @see it.eng.spago.dispatching.service.detail.IFaceDetailService#select(SourceBean, SourceBean)
     */
    public boolean select(SourceBean request, SourceBean response) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
            "AbstractDetailModule::select: metodo non implementato !");
        return false;
    } // public boolean select(SourceBean request, SourceBean response)

    /**
     * @see it.eng.spago.dispatching.service.detail.IFaceDetailService#update(SourceBean, SourceBean)
     */
    public boolean update(SourceBean request, SourceBean response) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
            "AbstractDetailModule::update: metodo non implementato !");
        return false;
    } // public boolean update(SourceBean request, SourceBean response)
} // public abstract class AbstractDetailModule extends AbstractModule implements IFaceDetailService
