/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event;

import java.util.Collection;
import java.util.Iterator;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.event.condition.EventConditionDriverManager;
import it.eng.spago.event.condition.EventConditionIFace;
import it.eng.spago.event.handler.EventHandlerDriverManager;
import it.eng.spago.event.handler.EventHandlerIFace;
import it.eng.spago.event.manager.EventsFacade;
import it.eng.spago.event.manager.EventsFacadeIFace;
import it.eng.spago.tracing.TracerSingleton;

public class EventExecutor {
    private EventExecutor() {
        super();
    } // private EventExecutor()

    public static void service(RequestContainer request, ResponseContainer response) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventExecutor::service: invocato");
        EventsFacadeIFace eventsFacade = new EventsFacade();

        /*
        EMFUserManager emfUserManager = EMFUserManager.getInstance(request);
        if (emfUserManager == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventExecutor::service: client non autenticato");
            return;
        } // if (emfUserManager == null)
        ManagerInterface manager = emfUserManager.getManager();
		*/

        Collection eventConditions = EventConditionDriverManager.getEventConditions(request, response);
        if (eventConditions == null)
            return;
        EventConditionsHandlers eventConditionsHandlers = new EventConditionsHandlers();
        Iterator eventConditionIterator = eventConditions.iterator();
        while (eventConditionIterator.hasNext()) {
            EventConditionIFace eventCondition = (EventConditionIFace)eventConditionIterator.next();
            String eventConditionKey = eventCondition.getKey();
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "EventExecutor::service: eventConditionKey [" + eventConditionKey + "]");
            Collection eventHandlerKeys = null;
            eventHandlerKeys = eventsFacade.getAllPluginsByEvent(eventConditionKey);
            if ((eventHandlerKeys == null) || (eventHandlerKeys.size() == 0))
                continue;
            Iterator eventHandlerKeyIterator = eventHandlerKeys.iterator();
            while (eventHandlerKeyIterator.hasNext()) {
                String eventHandlerKey = (String)eventHandlerKeyIterator.next();
                if (eventHandlerKey == null)
                    continue;
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "EventExecutor::service: eventHandlerKey [" + eventHandlerKey + "]");
                EventHandlerIFace eventHandler = (EventHandlerIFace)EventHandlerDriverManager.getEventHandler(eventHandlerKey);
                if (eventHandler != null)
                    eventConditionsHandlers.setEventConditionHandler(eventCondition, eventHandler);
            } // while (eventHandlerKeyIterator.hasNext())
        } // while (eventConditionIterator.hasNext())
        Collection eventConditionsHandlerRelations = eventConditionsHandlers.getEventConditionsHandlerRelations();
        if (eventConditionsHandlerRelations == null)
            return;
        Iterator eventConditionsHandlerRelationIterator = eventConditionsHandlerRelations.iterator();
        while (eventConditionsHandlerRelationIterator.hasNext()) {
            EventConditionsHandlerRelation eventConditionsHandlerRelation =
                (EventConditionsHandlerRelation)eventConditionsHandlerRelationIterator.next();
            EventHandlerIFace eventHandler = eventConditionsHandlerRelation.getEventHandler();
            eventHandler.service(eventConditionsHandlerRelation.getEventConditions(), request, response);
        } // while (eventConditionsHandlerRelationIterator.hasNext())
    } // public static void service(RequestContainer request, ResponseContainer response)
} // public class EventExecutor
