/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.handler;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

public class EventHandlerDriverManager implements InitializerIFace {
    private SourceBean _config = null;
    private static HashMap _eventHandlerDrivers = new HashMap();

    private EventHandlerDriverManager() {
        super();
        _config = null;
    } // private EventHandlerDriverManager()

    public void init(SourceBean config) {
        _config = config;
    } // public void init(SourceBean config)

    public SourceBean getConfig() {
        return _config;
    } // public SourceBean getConfig()

    public static void registerEventHandlerDriver(EventHandlerDriverIFace eventHandlerDriver) {
        if (eventHandlerDriver == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "EventHandlerDriverManager::registerEventHandlerDriver: eventHandlerDriver nullo");
            return;
        } // if (eventHandlerDriver == null)
        String eventHandlerKey = eventHandlerDriver.getEventHandler().getKey();
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "EventHandlerDriverManager::registerEventConditionDriver: eventHandlerKey [" + eventHandlerKey + "]");
        synchronized(EventHandlerDriverManager.class) {
            if (_eventHandlerDrivers.containsKey(eventHandlerKey)) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "EventHandlerDriverManager::registerEventConditionDriver: eventHandlerDriver gi registrato");
                return;
            } // if (_eventHandlerDrivers.containsKey(eventHandlerKey))
            _eventHandlerDrivers.put(eventHandlerKey, eventHandlerDriver);
        } // synchronized(EventHandlerDriverManager.class)
    } // public static void registerEventHandlerDriver(EventHandlerDriverIFace

    // eventHandlerDriver)
    public static EventHandlerIFace getEventHandler(String eventHandlerKey) {
        EventHandlerIFace eventHandler = null;
        if (eventHandlerKey == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "EventHandlerDriverManager::getEventHandler: eventHandlerKey nullo");
            return eventHandler;
        } // if (eventHandlerKey == null)
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "EventHandlerDriverManager::getEventHandler: eventHandlerKey [" + eventHandlerKey + "]");
        synchronized(EventHandlerDriverManager.class) {
            Collection eventHandlerDrivers = _eventHandlerDrivers.values();
            Iterator eventHandlerDriversIterator = eventHandlerDrivers.iterator();
            while (eventHandlerDriversIterator.hasNext()) {
                EventHandlerDriverIFace eventHandlerDriver = (EventHandlerDriverIFace)eventHandlerDriversIterator.next();
                if (eventHandlerDriver.acceptsEventHandler(eventHandlerKey)) {
                    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                        "EventHandlerDriverManager::getEventHandler: eventHandlerDriver trovato [" +
                        eventHandlerDriver.getClass().getName() + "]");
                    eventHandler = eventHandlerDriver.getEventHandler();
                    break;
                } // if (eventHandlerDriver.acceptsEventHandler(eventHandlerKey))
            } // while (eventHandlerDriversIterator.hasNext())
        } // synchronized(EventHandlerDriverManager.class)
        if (eventHandler == null)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "EventHandlerDriverManager::getEventHandler: nessun gestore trovato");
        return eventHandler;
    } // public static EventHandlerIFace getEventHandler(String eventHandlerKey)

    public static void deregisterEventHandlerDriver(EventHandlerDriverIFace eventHandlerDriver) {
        if (eventHandlerDriver == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "EventHandlerDriverManager::deregisterEventHandlerDriver: eventHandlerDriver nullo");
            return;
        } // if (eventHandlerDriver == null)
        String eventHandlerKey = eventHandlerDriver.getEventHandler().getKey();
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "EventHandlerDriverManager::deregisterEventHandlerDriver: eventHandlerKey [" + eventHandlerKey + "]");
        synchronized(EventHandlerDriverManager.class) {
            if (!(_eventHandlerDrivers.containsKey(eventHandlerKey))) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "EventHandlerDriverManager::deregisterEventHandlerDriver: eventHandlerDriver gi deregistrato");
                return;
            } // if (!(_eventHandlerDrivers.containsKey(EventConditionDriverName)))
            _eventHandlerDrivers.remove(eventHandlerKey);
        } // synchronized(EventHandlerDriverManager.class)
    } // public static void deregisterEventHandlerDriver(EventHandlerDriverIFace
    // eventHandlerDriver)
} // public class EventHandlerDriverManager implements InitializerIFace
