/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.handler;

import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

public class EventHandlerInitializer implements InitializerIFace {
    private SourceBean _config = null;

    public EventHandlerInitializer() {
        super();
        _config = null;
    } // public EventHandlerInitializer()

    public void init(SourceBean config) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventHandlerInitializer::init: config", config);
        _config = config;
        List eventHandlers = config.getAttributeAsList("HANDLER");
        for (int i = 0; i < eventHandlers.size(); i++) {
            try {
                SourceBean eventHandlerDefinition = (SourceBean)(eventHandlers.get(i));
                String eventHandlerDriverClassName = (String)(eventHandlerDefinition.getAttribute("DRIVER"));
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                    "EventHandlerInitializer::init: eventHandlerDriverClassName [" + eventHandlerDriverClassName + "]");
                Object eventHandlerDriver = Class.forName(eventHandlerDriverClassName).newInstance();
                SourceBean eventHandlerConfiguration = (SourceBean)eventHandlerDefinition.getAttribute("CONFIG");
                ((InitializerIFace)eventHandlerDriver).init(eventHandlerConfiguration);
                EventHandlerDriverManager.registerEventHandlerDriver((EventHandlerDriverIFace)eventHandlerDriver);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventHandlerInitializer::init:", ex);
            } // catch (Exception ex) try
        } // for (int i = 0; i < initializers.size(); i++)
    } // public void init(SourceBean config)

    public SourceBean getConfig() {
        return _config;
    } // public SourceBean getConfig()
} // public class EventHandlerInitializer implements InitializerIFace
