/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.manager;

import java.util.ArrayList;

import it.eng.spago.base.SourceBean;

/**
 * @author Daniela Butano
 * This class representing the facade to all classes manage events, plugin, triggers
 */
public class EventsFacade implements EventsFacadeIFace {
    protected EventManagerIFace _eventManager = null;
    protected PluginManagerIFace _pluginManager = null;
    protected TriggerManagerIFace _triggerManager = null;
    protected ProfileManager _profileManager = null;

    public EventsFacade() {
        _eventManager = new EventManager(this);
        _pluginManager = new PluginManager();
        _triggerManager = new TriggerManager(this);
        _profileManager = new ProfileManager();
    }

	/**
	 * @see it.eng.spago.event.TriggerMangerIFace#getAllPluginsByEvent(String)
	 */
    public ArrayList getAllPluginsByEvent(String eventName) {
        return _triggerManager.getAllPluginsByEvent(eventName);
    }

	/**
	 * @see it.eng.spago.event.TriggerManagerIFace#isUserTriggered(String, String, String)
	 */
    public boolean isUserTriggered(String userId, String eventName, String pluginName) {
        return _triggerManager.isUserTriggered(userId, eventName, pluginName);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#associatePluginToEvent(String, String)
	 */
    public void associatePluginToEvent(String pluginName, String eventName) throws Exception {
        _eventManager.associatePluginToEvent(pluginName, eventName);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getAllEvents()
	 */
    public SourceBean getAllEvents() throws Exception {
        return _eventManager.getAllEvents();
    }

	/**
	 * @see it.eng.spago.event.manager.PluginManagerIFace#getAllPlugins()
	 */
    public SourceBean getAllPlugins() throws Exception {
        return _pluginManager.getAllPlugins();
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getEventsForRole(String)
	 */
    public SourceBean getEventsForRole(String roleName) throws Exception {
        return _eventManager.getEventsForRole(roleName);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getEventsForType(String)
	 */
    public SourceBean getEventsForType(String typeEvent) throws Exception {
        return _eventManager.getEventsForType(typeEvent);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getEventsForUser(String)
	 */
    public SourceBean getEventsForUser(String userId) throws Exception {
        return _eventManager.getEventsForUser(userId);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getEventsType()
	 */
    public SourceBean getEventsType() throws Exception {
        return _eventManager.getEventsType();
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getPluginAssociableWithEvent(String, String)
	 */
    public SourceBean getPluginAssociableWithEvent(String eventName, String pluginType) throws Exception {
        return _eventManager.getPluginAssociableWithEvent(eventName, pluginType);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getPluginsAssociableWithEvent(String)
	 */
    public SourceBean getPluginsAssociableWithEvent(String eventName) throws Exception {
        return _eventManager.getPluginsAssociableWithEvent(eventName);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#removePluginFromEvent(String, String)
	 */
    public void removePluginFromEvent(String pluginName, String eventName) throws Exception {
        _eventManager.removePluginFromEvent(pluginName, eventName);
    }

	/**
	 * @see it.eng.spago.event.manager.PluginManagerIFace#getPluginsForType(String)
	 */
    public SourceBean getPluginsForType(String typePlugin) throws Exception {
        return _pluginManager.getPluginsForType(typePlugin);
    }

	/**
	 * @see it.eng.spago.event.manager.PluginManagerIFace#getPluginType()
	 */
    public SourceBean getPluginType() throws Exception {
        return _pluginManager.getPluginType();
    }

	/**
	 * @see it.eng.spago.event.manager.PluginManagerIFace#createNewPlugin(String, String, String)
	 */
    public void createNewPlugin(String key, String description, String type) throws Exception {
        _pluginManager.createNewPlugin(key, description, type);
    }

	/**
	 * @see it.eng.spago.event.manager.PluginManagerIFace#removePlugin(String)
	 */
    public void removePlugin(String key) throws Exception {
        _pluginManager.removePlugin(key);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#createNewEventType(String, String)
	 */
    public void createNewEventType(String key, String description) throws Exception {
        _eventManager.createNewEventType(key, description);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#createNewEvent(String, String, String)
	 */
    public void createNewEvent(String key, String description, String type) throws Exception {
        _eventManager.createNewEvent(key, description, type);
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#getAllTriggers()
	 */
    public SourceBean getAllTriggers() throws Exception {
        return _triggerManager.getAllTriggers();
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#getAllTriggers(String, String, String, String)
	 */
    public SourceBean getAllTriggers(String event, String plugin, String idProfile, String typeProfile) throws Exception {
        return _triggerManager.getAllTriggers(event, plugin, idProfile, typeProfile);
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#getAllTriggersByEvent(String)
	 */
    public SourceBean getAllTriggersByEvent(String eventName) throws Exception {
        return _triggerManager.getAllTriggersByEvent(eventName);
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#getAllTriggersByEventPlugin(String, String)
	 */
    public SourceBean getAllTriggersByEventPlugin(String eventName, String pluginName) throws Exception {
        return _triggerManager.getAllTriggersByEventPlugin(eventName, pluginName);
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#getAllUsersTriggered(String, String)
	 */
    public ArrayList getAllUsersTriggered(String eventName, String pluginName) throws Exception {
        return _triggerManager.getAllUsersTriggered(eventName, pluginName);
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#removeTrigger(String, String, String, String)
	 */
    public void removeTrigger(String event, String plugin, String idProfile, String typeProfile) throws Exception {
        _triggerManager.removeTrigger(event, plugin, idProfile, typeProfile);
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#getAllTriggersByPlugin(String)
	 */
    public SourceBean getAllTriggersByPlugin(String pluginName) throws Exception {
        return _triggerManager.getAllTriggersByPlugin(pluginName);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#getRolesForEvent(String)
	 */
    public SourceBean getRolesForEvent(String eventName) throws Exception {
        return _eventManager.getRolesForEvent(eventName);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#associateRoleToEvent(String, String)
	 */
    public void associateRoleToEvent(String roleName, String eventName) throws Exception {
        _eventManager.associateRoleToEvent(roleName, eventName);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#removeRoleFromEvent(String, String)
	 */
    public void removeRoleFromEvent(String roleName, String eventName) throws Exception {
        _eventManager.removeRoleFromEvent(roleName, eventName);
    }

	/**
	 * @see it.eng.spago.event.manager.PluginManagerIFace#createNewPluginType(String, String)
	 */
    public void createNewPluginType(String key, String description) throws Exception {
        _pluginManager.createNewPluginType(key, description);
    }

	/**
	 * @see it.eng.spago.event.manager.PluginManagerIFace#removePluginType(String)
	 */
    public void removePluginType(String key) throws Exception {
        _pluginManager.removePluginType(key);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#removeEventType(String)
	 */
    public void removeEventType(String key) throws Exception {
        _eventManager.removeEventType(key);
    }

	/**
	 * @see it.eng.spago.event.manager.TriggerManagerIFace#createNewTrigger(String, String, String, String)
	 */
    public void createNewTrigger(String event, String plugin, String idProfile, String typeProfile) throws Exception {
        _triggerManager.createNewTrigger(event, plugin, idProfile, typeProfile);
    }

	/**
	 * @see it.eng.spago.event.manager.EventManagerIFace#removeEvent(String)
	 */
    public void removeEvent(String key) throws Exception {
        _eventManager.removeEvent(key);
    }

	/**
	 * @see it.eng.spago.event.manager.ProfileManagerIFace#getAllRoles()
	 */
    public ArrayList getAllRoles() throws Exception {
        return _profileManager.getAllRoles();
    }

	/**
	 * @see it.eng.spago.event.manager.ProfileManagerIFace#getAllRoles(String)
	 */
    public ArrayList getAllRoles(String userId) throws Exception {
        return _profileManager.getAllRoles(userId);
    }

	/**
	 * @see it.eng.spago.event.manager.ProfileManagerIFace#getAllUsers()
	 */
    public ArrayList getAllUsers() throws Exception {
        return _profileManager.getAllUsers();
    }

	/**
	 * @see it.eng.spago.event.manager.ProfileManagerIFace#getAllUsers(String)
	 */
    public ArrayList getAllUsers(String roleId) throws Exception {
        return _profileManager.getAllUsers(roleId);
    }
}
