/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.manager;

import java.util.ArrayList;

import it.eng.spago.base.SourceBean;

/**
 * @author Daniela Butano
 * This class representing the facade to all classes manage events, plugin, triggers
 */
public interface EventsFacadeIFace extends EventManagerIFace, PluginManagerIFace, TriggerManagerIFace, ProfileManagerIFace {
	/***************************************************
	 *            see EventManager                     *
	 * *************************************************/
    public SourceBean getAllEvents() throws Exception;

    public SourceBean getEventsForType(String typeEvent) throws Exception;

    public SourceBean getPluginsAssociableWithEvent(String eventName) throws Exception;

    public SourceBean getPluginAssociableWithEvent(String eventName, String pluginType) throws Exception;

    public void associatePluginToEvent(String pluginName, String eventName) throws Exception;

    public void removePluginFromEvent(String pluginName, String eventName) throws Exception;

    public SourceBean getEventsType() throws Exception;

    public SourceBean getEventsForRole(String roleName) throws Exception;

    public SourceBean getEventsForUser(String userId) throws Exception;

    public SourceBean getRolesForEvent(String eventName) throws Exception;

    public void removeRoleFromEvent(String roleName, String eventName) throws Exception;

    public void associateRoleToEvent(String roleName, String eventName) throws Exception;

    public void createNewEventType(String key, String description) throws Exception;

    public void removeEventType(String key) throws Exception;

    public void createNewEvent(String key, String description, String type) throws Exception;

    public void removeEvent(String key) throws Exception;

    /***************************************************
	 *            see PluginManager                    *
	 * *************************************************/
    public SourceBean getAllPlugins() throws Exception;

    public SourceBean getPluginsForType(String typePlugin) throws Exception;

    public SourceBean getPluginType() throws Exception;

    public void createNewPlugin(String key, String description, String type) throws Exception;

    public void removePlugin(String key) throws Exception;

    public void createNewPluginType(String key, String description) throws Exception;

    public void removePluginType(String key) throws Exception;

	/***************************************************
	 *            see TriggerManager                   *
	 * *************************************************/
    public SourceBean getAllTriggers() throws Exception;

    public SourceBean getAllTriggersByEvent(String eventName) throws Exception;

    public SourceBean getAllTriggersByPlugin(String pluginName) throws Exception;

    public SourceBean getAllTriggersByEventPlugin(String eventName, String pluginName) throws Exception;

    public SourceBean getAllTriggers(String event, String plugin, String idProfile, String typeProfile) throws Exception;

    public ArrayList getAllPluginsByEvent(String eventName);

    public boolean isUserTriggered(String userId, String eventName, String pluginName);

    public ArrayList getAllUsersTriggered(String eventName, String pluginName) throws Exception;

    public void removeTrigger(String event, String plugin, String idProfile, String typeProfile) throws Exception;

    public void createNewTrigger(String event, String plugin, String idProfile, String typeProfile) throws Exception;

	/***************************************************
	 *            see ProfileManager                   *
	 * *************************************************/
    public ArrayList getAllUsers() throws Exception;

    public ArrayList getAllUsers(String roleId) throws Exception;

    public ArrayList getAllRoles() throws Exception;

    public ArrayList getAllRoles(String userId) throws Exception;
}
