/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.smart;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;

/**
 * La classe <code>AbstractRowHandler</code> estende la classe
 * <code>AbstractRowProvider</code> ed aggiunge i metodi per eseguire istruzioni
 * di <em>insert</em> e <em>delete</em> di una riga ele istruzioni di
 * <em>select</em> e <em>update</em> dei dati di dettaglio di una riga.
 * <p>
 * @version 1.0, 15/03/2003
 */
public abstract class AbstractRowHandler extends AbstractRowProvider implements IFaceRowHandler {
    /**
     * Costruisce un <code>AbstractRowProvider</code>. Questo costruttore
     * &egrave; vuoto ed ogni azione di inizializzazione &egrave; demandata
     * al metodo <code>init(SourceBean config)</code> in
     * <code>AbstractRowProvider</code>.
     * <p>
     * @see AbstractRowHandler#init(SourceBean)
     */
    public AbstractRowHandler() {
        super();
    } // public AbstractRowHandler()

    /**
     * Esegue l'inserimento di una riga nella lista. Ritorna l'esito
     * dell'operazione.
     * <p>
     * @return <code>boolean</code> esito dell'operazione
     */
    public boolean insertRow() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "AbstractRowProvider::insertRow: metodo non implementato !");
        return false;
    } // public boolean insertRow()

    /**
     * Esegue l'operazione di update dei dati di dettaglio relativi all'ultima
     * riga recuperata o individuata con il comando <code>absolute(int).
     */
    public boolean updateRow() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "AbstractRowProvider::updateRow: metodo non implementato !");
        return false;
    } // public boolean updateRow()

    /**
     * Esegue l'operazione di select dei dati di dettaglio relativi all'ultima
     * riga recuperata o individuata con il comando <code>absolute(int).
     * <p>
     * @return <code>SourceBean</code> che contiene i dati richiesti.
     */
    public SourceBean selectRow() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "AbstractRowProvider::selectRow: metodo non implementato !");
        return null;
    } // public SourceBean selectRow()

    /**
     * Cancella l'ultima riga recuperata o individuata con il comando
     * <code>absolute(int).
     * <p>
     * @return <code>boolean</code> esito dell'operazione
     */
    public boolean deleteRow() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "AbstractRowProvider::deleteRow: metodo non implementato !");
        return false;
    } // public boolean deleteRow()
} // public abstract class AbstractRowHandler extends AbstractRowProvider implements IFaceRowHandler
