/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.smart;

import java.util.Collection;

public interface IFacePageProvider {
    public static final int LAST = -1;

    /**
     * Notifica al componente di ricaricare i dati dei quali fa cacheing.
     * <p>
     * @see AbstractPageProvider#init(SourceBean)
     */
    void reload();

    /**
     * Restituisce il <em>row provider</em> associato.
     * @return <code>IFaceRowProvider</code>
     * @see IFaceRowProvider
     */
    IFaceRowProvider getRowProvider();

    /**
     * Imposta il <em>row provider</em> associato.
     * @param rowProvider <code>IFaceRowProvider</code>
     * @see IFaceRowProvider
     */
    void setRowProvider(IFaceRowProvider rowProvider);

    /**
     * Ritorna il numero di righe per pagina.
     * @return <code>int</code>
     */
    int getPageSize();

    /**
     * Imposta il numero di righe per pagina.
     * @param size <code>int</code>
     */
    void setPageSize(int size);

    /**
     * Ritorna il numero di pagine previste.
     * @return <code>int</code>
     */
    int pages();

    /**
     * Ritorna una collezione con tutte le righe che formano la pagina di indice
     * <em>page</em> richiesto.
     * @param page <code>int</code>
     * @return <code>Vector</code>
     */
    Collection getPage(int page);

    int getCurrentPage();
} // public interface IFacePageProvider
