/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.smart;

public interface IFaceRowProvider {
    public static final int LAST = -1;

    /**
     * Notifica al componente di ricaricare i dati dei quali fa cacheing.
     * <p>
     * @see AbstractRowtProvider#init(SourceBean)
     */
    void reload();

    /**
     * Questo metodo serve per notificare al componente l'inizio delle
     * operazioni di lettura.
     */
    void open();

    /**
     * Questo metodo serve per posizionare il <em>cursore</em> di lettura
     * alla riga individuata da <em>row</em>.
     * <p>
     * @param row <code>int</code> il numero della riga su cui posizionarsi.
     */
    void absolute(int row);

    /**
     * Ritorna un <code>Object</code> che rappresenta la riga individuata da
     * <em>row</em>.
     * <p>
     * @return <code>Object</code> la riga individuata da <em>row</em>.
     * @param row <code>int</code> il numero della riga da ritornare.
     */
    Object getRow(int row);

    /**
     * Ritorna un <code>Object</code> che rappresenta la riga successiva
     * all'ultima recuperata o individuata con il comando <code>absolute(int)
     * </code>.
     * <p>
     * @return <code>Object</code> la riga individuata da <em>row</em>.
     * @param row <code>int</code> il numero della riga da ritornare.
     */
    Object getNextRow();

    int getCurrentRow();

    /**
     * Questo metodo serve per notificare al componente la fine delle operazioni
     * di lettura.
     */
    void close();

    /**
     * Ritorna il numero di righe fornite dal <em>row provider</em>.
     * <p>
     * @return <code>int</code>
     */
    int rows();
} // public interface IFaceRowProvider
