/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.paginator.smart.impl;

import it.eng.spago.base.SourceBean;
import it.eng.spago.paginator.smart.IFaceRowHandler;
import it.eng.spago.util.QueryExecutor;

/**
 * La classe <code>DBRowHandler</code> estende <code>AbstractRowHandler</code>
 * ed implementa i metodi di accesso alle sorgenti di dati riconducibili al
 * pacchetto dbaccess del framework.
 * <p>
 * @version 1.0, 15/03/2003
 */
public class DBRowHandler extends DBRowProvider implements IFaceRowHandler {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
     * Costruisce un <code>DBRowProvider</code>. Questo costruttore &egrave;
     * vuoto ed ogni azione di inizializzazione &egrave; demandata al metodo
     * <code>init(SourceBean config)</code>.
     * <p>
     * @see DBRowHandler#init(SourceBean)
     */
    public DBRowHandler() {
        super();
    } // public DBRowHandler()

    /**
     * Cancella l'ultima riga recuperata o individuata con il comando
     * <code>absolute(int).
     * <p>
     * @return <code>boolean</code> esito dell'operazione
     */
    public boolean deleteRow() {
        SourceBean config = getConfig();
        SourceBean query = (SourceBean)config.getAttribute("DELETE_QUERY");
        String pool = (String)config.getAttribute("POOL");
        return ((Boolean)(QueryExecutor.executeQuery(getRequestContainer(), getResponseContainer(),
            pool, query, "DELETE"))).booleanValue();
    } // public boolean deleteRow()

    /**
     * Esegue l'inserimento di una riga nella lista. Ritorna l'esito
     * dell'operazione.
     * <p>
     * @return <code>boolean</code> esito dell'operazione
     */
    public boolean insertRow() {
        SourceBean config = getConfig();
        SourceBean query = (SourceBean)config.getAttribute("INSERT_QUERY");
        String pool = (String)config.getAttribute("POOL");
        return ((Boolean)(QueryExecutor.executeQuery(getRequestContainer(), getResponseContainer(),
            pool, query, "INSERT"))).booleanValue();
    } // public boolean insertRow()

    /**
     * Esegue l'operazione di select dei dati di dettaglio relativi all'ultima
     * riga recuperata o individuata con il comando <code>absolute(int).
     * <p>
     * @return <code>SourceBean</code> che contiene i dati richiesti.
     */
    public SourceBean selectRow() {
        SourceBean config = getConfig();
        SourceBean query = (SourceBean)config.getAttribute("SELECT_QUERY");
        String pool = (String)config.getAttribute("POOL");
        return (SourceBean)(QueryExecutor.executeQuery(getRequestContainer(), getResponseContainer(), pool, query, "SELECT"));
    } // public SourceBean selectRow()

    /**
     * Esegue l'operazione di update dei dati di dettaglio relativi all'ultima
     * riga recuperata o individuata con il comando <code>absolute(int). I dati
     * da modificare vengono recuperati come di consueto dal contesto attraverso
     * i parametri dello statement fornito in fase di configurazione.
     * <p>
     * @return <code>boolean</code> esito dell'operazione
     * @see QueryExecutor#executeQuery(RequestContainer,
     * ResponseContainer, String, SourceBean, String)
     */
    public boolean updateRow() {
        SourceBean config = getConfig();
        SourceBean query = (SourceBean)config.getAttribute("UPDATE_QUERY");
        String pool = (String)config.getAttribute("POOL");
        return ((Boolean)(QueryExecutor.executeQuery(getRequestContainer(), getResponseContainer(),
            pool, query, "UPDATE"))).booleanValue();
    } // public boolean updateRow()
} // public class DBRowHandler extends DBRowProvider implements IFaceRowHandler
