/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.presentation;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.xml.transform.Templates;
import javax.xml.transform.stream.StreamResult;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.transcoding.Transcoder;

public class PresentationRendering {
    public static String render(
        ResponseContainer responseContainer,
        ArrayList resources) {
        String toRender = null;
        try {
            SourceBean responseBean = new SourceBean("RESPONSE");
            responseBean.setAttribute(responseContainer.getServiceResponse());
            responseBean.setAttribute(
                responseContainer.getErrorHandler().getSourceBean());
            toRender = responseBean.toXML();
        } // try
        catch (SourceBeanException sbe) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "PresentationRendering::render: errore creazione stream RESPONSE",
                sbe);
            return "";
        } // catch (SourceBeanException sbe)
        if ((resources == null) || (resources.size() == 0)) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "PresentationRendering::render: resources nullo");
            return toRender;
        } // if ((resources == null) || (resources.size() == 0))
        //Modifica Monica 15/01/2004 - inizio
        //recupero il SourceBean della busta ITEM, contenente la busta CONFIG
        String stylesheetName = (String) ((SourceBean) resources.get(0)).getAttribute("resource");
        //Modifica Monica 15/01/2004 - fine        

        if (stylesheetName == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "PresentationRendering::render: stylesheetName nullo");
            return toRender;
        } // if (stylesheetName == null)
        ApplicationContainer application = ApplicationContainer.getInstance();
        Hashtable xslTemplates =
            (Hashtable) application.getAttribute("XSL_TEMPLATES");
        if (xslTemplates == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "PresentationRendering::render: xslTemplates nullo");
            return toRender;
        } // if (xslTemplates == null)
        // Recupero dalla cache il file contrassegnato dalla label stylesheetLabel
        StylesheetFile stylesheet =
            (StylesheetFile) xslTemplates.get(stylesheetName);
        if (stylesheet == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "PresentationRendering::render: template di stylesheetName ["
                    + stylesheetName
                    + "] non trovato");
            return toRender;
        } // if (stylesheet == null)
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "PresentationRendering::render: template di stylesheetName ["
                + stylesheetName
                + "]");
        // Recupero la struttura XSL dall'oggetto StylesheetFile
        Templates template = stylesheet.getTemplate();
        ByteArrayOutputStream byteArrayOutputStream =
            new ByteArrayOutputStream();
        StreamResult streamResult = new StreamResult(byteArrayOutputStream);
        Transcoder.perform(toRender, template, streamResult);
        String result = byteArrayOutputStream.toString();
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "PresentationRendering::render: risultato trasformazione\n"
                + result);
        return result;
    } // public static String render(ResponseContainer responseContainer, ArrayList resources)
} // public class PresentationRendering
