/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.presentation;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.navigation.Navigator;
import it.eng.spago.security.SecurityException;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

import java.util.ArrayList;
import java.util.List;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 										    
**/
/**
 * @author 
 */
public class Publisher {
    private String _name = null;
    private String _channel = null;
    private String _type = null;
    private String _mode = null;
    private ArrayList _resources = null;

    public Publisher() {
        super();
        _name = null;
        _channel = null;
        _type = Constants.NOTHING_PUBLISHER_TYPE;
        _mode = null;
        _resources = new ArrayList();
    } // public Publisher()

    public String getName() {
        return _name;
    } // public String getName()

    public void setName(String name) {
        _name = name;
    } // public void setName(String name)

    public String getChannel() {
        return _channel;
    } // public String getChannel()

    public void setChannel(String channel) {
        _channel = channel;
    } // public void setChannel(String channel)

    public String getType() {
        return _type;
    } // public String getType()

    public void setType(String type) {
        if (type == null)
            _type = Constants.NOTHING_PUBLISHER_TYPE;
        else
            _type = type;
    } // public void setType(String type)

    public String getMode() {
        return _mode;
    } // public String getMode()

    public void setMode(String mode) {
        _mode = mode;
    } // public void setMode(String mode)

    public ArrayList getResources() {
        return _resources;
    } // public ArrayList getResources()

    //Modifica Monica 15/01/2004 - inizio
    //ArrayList di SourceBean anzich di String
    public void addResource(SourceBean resource) {
        if (resource == null)
            return;
        _resources.add(resource);
    } // public void addResource(String resource)
    //Modifica Monica 15/01/2004 - fine

    public static Publisher getPublisher(
        RequestContainer requestContainer,
        ResponseContainer responseContainer,
        Exception serviceException) {
        String publisherName = null;
        if (serviceException != null)
            if (serviceException instanceof SecurityException)
                publisherName = "SECURITY_ERROR_PUBLISHER";
            else
                publisherName = "SERVICE_ERROR_PUBLISHER";
        else {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "Publisher::getPublisher: invocato");
            String businessType = responseContainer.getBusinessType();
            
            if (businessType == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "Publisher::getPublisher: businessType nullo");
                return null;
            } // if (businessType == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "Publisher::getPublisher: businessType [" + businessType + "]");
            String businessName = responseContainer.getBusinessName();
            if (businessName == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "Publisher::getPublisher: businessName nullo");
                return null;
            } // if (businessName == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "Publisher::getPublisher: businessName [" + businessName + "]");
            ConfigSingleton configSingleton = ConfigSingleton.getInstance();
            Object mappingsObject =
                configSingleton.getFilteredSourceBeanAttribute(
                    "PRESENTATION.MAPPING",
                    "BUSINESS_NAME",
                    businessName);
            if (mappingsObject == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "Publisher::getPublisher: mapping non trovato");
                return null;
            } // if (mappingsObject == null)
            List mappings = null;
            if (mappingsObject instanceof SourceBean) {
                mappings = new ArrayList(1);
                mappings.add(mappingsObject);
            } // if (mappingsObject instanceof SourceBean)
            else
                mappings = (List) mappingsObject;
            
            //Take the first mapping
            for (int i = 0; i < mappings.size(); i++) {
                SourceBean mapping = (SourceBean) mappings.get(i);
                String localBusinessType =
                    (String) mapping.getAttribute("BUSINESS_TYPE");
                if ((localBusinessType != null)
                    && (localBusinessType.equalsIgnoreCase(businessType))) {
                    publisherName =
                        (String) mapping.getAttribute("PUBLISHER_NAME");
                    break;
                } // if ((localBusinessName != null) &&
                // (localBusinessName.equalsIgnoreCase(businessName)))
            } // for (int i = 0; i < mappings.size(); i++)
        } // if (serviceError)
        Publisher publisher =
            getPublisher(requestContainer, responseContainer, publisherName);
        while ((publisher != null)
            && (publisher.getType() != null)
            && (publisher.getType().equalsIgnoreCase("JAVA"))) {
            ArrayList resources = publisher.getResources();
            if (resources.size() == 0) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "Publisher::getPublisher: classe del publisher ["
                        + publisherName
                        + "] non specificata");
                return null;
            } // if (resources.size() == 0)

            //Modifica Monica 15/01/2004 - inizio
            //recupero il SourceBean della busta ITEM, contenente la busta CONFIG
            String publisherDispatcherClass =
                (String) ((SourceBean) resources.get(0)).getAttribute(
                    "resource");
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "Publisher::getPublisher: publisherDispatcherClass ["
                    + publisherDispatcherClass
                    + "]");
            PublisherDispatcherIFace publisherDispatcher = null;
            try {
                publisherDispatcher =
                    (PublisherDispatcherIFace) Class
                        .forName(publisherDispatcherClass)
                        .newInstance();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "Publisher::getPublisher: classe del publisher non valida",
                    ex);
                return null;
            } // catch (Exception ex) try
            if (publisherDispatcher instanceof InitializerIFace) {
                SourceBean bustaConfig =
                    (SourceBean) ((SourceBean) resources.get(0)).getAttribute(
                        "CONFIG");
                ((InitializerIFace) publisherDispatcher).init(bustaConfig);
            }
            //              Modifica Monica 15/01/2004 - fine
            publisherName =
                publisherDispatcher.getPublisherName(
                    requestContainer,
                    responseContainer);
            publisher =
                getPublisher(
                    requestContainer,
                    responseContainer,
                    publisherName);
        } // while ((publisher != null) && (publisher.getType() != null)
        // && (publisher.getType().equalsIgnoreCase("JAVA")))
        return publisher;
    } // public static Publisher getPublisher(RequestContainer requestContainer, ResponseContainer responseContainer, boolean serviceError)

    
    public static String getLogicalUserAgent(RequestContainer requestContainer){
    	
    	String originalUserAgentString = null;
    	if (requestContainer.getAttribute("user-agent") != null){
    		originalUserAgentString = (String)requestContainer.getAttribute("user-agent");
    	}
    	
    	
    	if (originalUserAgentString == null){
    		TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "Publisher::getLogicalUserAgent:  + originalUserAgentString nullo");
    		return null;
    	}
    	
    	SourceBean userAgents = (SourceBean)ConfigSingleton.getInstance().getAttribute("USER-AGENTS");
    	
    	TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "Publisher::getLogicalUserAgent: Cerco di mappare lo user agent [" + originalUserAgentString+ "] su uno user agent logico");
    	
    	if (userAgents != null){
    		Object resultingUserAgent = userAgents.getFilteredSourceBeanAttribute("USER-AGENT", "identifierString", originalUserAgentString);
    		if (resultingUserAgent  instanceof List) {
				
    			TracerSingleton.log(
    	                Constants.NOME_MODULO,
    	                TracerSingleton.CRITICAL,
    	                "Publisher::getLogicalUserAgent: Trovato piu di un user agent logico per user agent [" + originalUserAgentString+ "]");
    	            return null;
			}else{
				return (String)((SourceBean)resultingUserAgent).getAttribute("logicalName");
			}
    	}else{
    		TracerSingleton.log(
	                Constants.NOME_MODULO,
	                TracerSingleton.WARNING,
	                "Publisher::getLogicalUserAgent: BUSTA USER-AGENTS non trovata nel Configuratore");
    		return null;
    	}
    }
    
    
    private static Publisher getPublisher(
        RequestContainer requestContainer,
        ResponseContainer responseContainer,
        String publisherName) {
        if (publisherName == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "Publisher::getPublisher: mapping non trovato");
            return null;
        } // if (publisherName == null)
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "Publisher::getPublisher: publisherName [" + publisherName + "]");
        ConfigSingleton configSingleton = ConfigSingleton.getInstance();
        SourceBean publisherConfig =
            (SourceBean) configSingleton.getFilteredSourceBeanAttribute(
                "PUBLISHERS.PUBLISHER",
                "NAME",
                publisherName);
        if (publisherConfig == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "Publisher::getPublisher: publisher non trovato");
            return null;
        } // if (publisherConfig == null)
        String channelType = requestContainer.getChannelType();
        
        if (channelType == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "Publisher::getPublisher: channelType nullo");
            return null;
        } // if (channelType == null)
        
        String logicalUserAgentOfRequest = getLogicalUserAgent(requestContainer);
        
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "Publisher::getPublisher: channelType [" + channelType + "]");
        
        TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "Publisher::getPublisher: logicalUserAgent [" + logicalUserAgentOfRequest + "]");
        
        // 28-11-2005 Andrea Zoppello --> Handling user Agent
        Object renderingConfigObject = publisherConfig.getFilteredSourceBeanAttribute(
                    "RENDERING",
                    "CHANNEL",
                    channelType);
        
        SourceBean renderingConfig = null;
        
        if ( renderingConfigObject instanceof List) {
			
        	SourceBean tmpRenderConfig = null;
        	String logicalUserAgentOfRenderConfig = null;
        	if (logicalUserAgentOfRequest != null){
        		for (int i=0; i < ((List)renderingConfigObject).size(); i++){
        			tmpRenderConfig = (SourceBean)((List)renderingConfigObject).get(i);
        			logicalUserAgentOfRenderConfig = (String)tmpRenderConfig.getAttribute("user-agent");
        			if ((logicalUserAgentOfRenderConfig != null) && 
        					(logicalUserAgentOfRenderConfig.equalsIgnoreCase(logicalUserAgentOfRequest))){
        					renderingConfig = tmpRenderConfig;
        					break;
        			}//if ((logicalUserAgentOfRenderConfig != null) && (logicalUserAgentOfRenderConfig.equalsIgnoreCase(logicalUserAgentOfRequest))){
        		}//for (int i=0; i < ((List)renderingConfigObject).size(); i++){
        	}//if (logicalUserAgentOfRequest != null){
        	
        	// Listing all the rendering config section noone matches with the user agent attribute
        	if (renderingConfig == null){
        		renderingConfig = (SourceBean)((List)renderingConfigObject).get(0);
        	}
		}else{
			renderingConfig = (SourceBean)renderingConfigObject;
		}
        
//      28-11-2005 Andrea Zoppello --> Fine Modifiche
        if (renderingConfig == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "Publisher::getPublisher: publisher non trovato per il canale ["
                    + channelType
                    + "]");
            return null;
        } // if (renderingConfig == null)
        Publisher publisher = new Publisher();
        publisher.setName(publisherName);
        publisher.setChannel(channelType);
        publisher.setType((String) renderingConfig.getAttribute("TYPE"));
        if (publisher
            .getType()
            .equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE)) {
            try {
                SourceBean loopbackServiceRequest =
                    new SourceBean(Constants.SERVICE_REQUEST);
                loopbackServiceRequest.setAttribute(
                    Navigator.NAVIGATOR_DISABLED,
                    "TRUE");
                List resourcesConfig =
                    renderingConfig.getAttributeAsList("RESOURCES.PARAMETER");
                for (int j = 0; j < resourcesConfig.size(); j++) {
                    SourceBean consequence =
                        (SourceBean) resourcesConfig.get(j);
                    String parameterName =
                        (String) consequence.getAttribute("NAME");
                    String parameterScope =
                        (String) consequence.getAttribute("SCOPE");
                    String parameterType =
                        (String) consequence.getAttribute("TYPE");
                    String parameterValue =
                        (String) consequence.getAttribute("VALUE");
                    Object inParameterValue = null;
                    if (parameterType.equalsIgnoreCase("ABSOLUTE"))
                        inParameterValue = parameterValue;
                    else
                        inParameterValue =
                            ContextScooping.getScopedParameter(
                                requestContainer,
                                responseContainer,
                                parameterValue,
                                parameterScope);
                    if (inParameterValue == null)
                        continue;
                    if (inParameterValue instanceof SourceBean)
                        loopbackServiceRequest.setAttribute(
                            (SourceBean) inParameterValue);
                    else
                        loopbackServiceRequest.setAttribute(
                            parameterName,
                            inParameterValue);
                } // for (int j = 0; j < consequences.size(); j++)
                responseContainer.setLoopbackServiceRequest(
                    loopbackServiceRequest);
            } // try
            catch (SourceBeanException sbe) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "Publisher::getPublisher:",
                    sbe);
            } // catch (SourceBeanException sbe)
        } // if (publisher.getType().equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE))
        else {
            List resourcesConfig =
                renderingConfig.getAttributeAsList("RESOURCES.ITEM");
            for (int i = 0; i < resourcesConfig.size(); i++) {
                SourceBean resourceConfig = (SourceBean) resourcesConfig.get(i);
                //Modifica Monica 15/01/2004 - inizio                     
                //                    String resource = (String)resourceConfig.getAttribute("RESOURCE");
                //                    if (resource != null)
                publisher.addResource(resourceConfig);
                //Modifica Monica 15/01/2004 - fine    
            } // for (int i = 0; i < resourcesConfig.size(); i++)
        } // if ((publisher.getType() != null) && (publisher.getType().equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE))) else
        publisher.setMode((String) renderingConfig.getAttribute("MODE"));
        return publisher;
    } // private static Publisher getPublisher(RequestContainer
    // requestContainer, ResponseContainer responseContainer, String publisherName)
} // public class Publisher
