/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.util;

import java.sql.Types;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.InformationDataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.tracing.TracerSingleton;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 										    
**/

/**
 * @author 
 */
public abstract class QueryExecutor {
    public static final String CREATE = "CREATE";
    public static final String READ = "READ";
    public static final String UPDATE = "UPDATE";
    public static final String DELETE = "DELETE";

    public static Object executeQuery(
        RequestContainer requestContainer,
        ResponseContainer responseContainer,
        String pool,
        SourceBean query,
        String type) {
        Object result = null;
        DataConnectionManager dataConnectionManager = null;
        DataConnection dataConnection = null;
        SQLCommand sqlCommand = null;
        DataResult dataResult = null;
        try {
            dataConnectionManager = DataConnectionManager.getInstance();
            dataConnection = dataConnectionManager.getConnection(pool);
            String statement =
                SQLStatements.getStatement(
                    (String) query.getAttribute("STATEMENT"));
            if (type.equalsIgnoreCase("INSERT"))
                sqlCommand = dataConnection.createInsertCommand(statement);
            else if (type.equalsIgnoreCase("UPDATE"))
                sqlCommand = dataConnection.createUpdateCommand(statement);
            else if (type.equalsIgnoreCase("DELETE"))
                sqlCommand = dataConnection.createDeleteCommand(statement);
            else
                sqlCommand = dataConnection.createSelectCommand(statement);
            ArrayList inputParameters = new ArrayList();
            List parameters = query.getAttributeAsList("PARAMETER");
            for (int i = 0; i < parameters.size(); i++) {
                SourceBean parameter = (SourceBean) parameters.get(i);
                String parameterType = (String) parameter.getAttribute("TYPE");
                String parameterValue =
                    (String) parameter.getAttribute("VALUE");
                String parameterScope =
                    (String) parameter.getAttribute("SCOPE");
                String inParameterValue = null;
                if (parameterType.equalsIgnoreCase("ABSOLUTE"))
                    inParameterValue = parameterValue;
                else {
                    Object parameterValueObject =
                        ContextScooping.getScopedParameter(
                            requestContainer,
                            responseContainer,
                            parameterValue,
                            parameterScope);
                    if (parameterValueObject != null)
                        inParameterValue = parameterValueObject.toString();
                } // if (parameterType.equalsIgnoreCase("ABSOLUTE")) else
                if (inParameterValue == null)
                    inParameterValue = "";
                inputParameters.add(
                    dataConnection.createDataField(
                        "",
                        Types.VARCHAR,
                        inParameterValue));
            } // for (int i = 0; i < fields.size(); i++)
            dataResult = sqlCommand.execute(inputParameters);
            if (type.equalsIgnoreCase("SELECT")) {
                ScrollableDataResult scrollableDataResult =
                    (ScrollableDataResult) dataResult.getDataObject();
                result = scrollableDataResult.getSourceBean();
            } // if (type.equalsIgnoreCase("SELECT"))
            else {
                InformationDataResult informationDataResult =
                    (InformationDataResult) dataResult.getDataObject();
                result =
                    new Boolean(informationDataResult.getAffectedRows() > 0);
            } // if (type.equalsIgnoreCase("SELECT")) else
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "QueryExecutor::executeQuery:",
                ex);
        } // catch (Exception ex) try
        finally {
            Utils.releaseResources(dataConnection, sqlCommand, dataResult);
        } // finally try
        return result;
    } //     public static Object executeQuery(RequestContainer

    // requestContainer, ResponseContainer responseContainer, String pool,
    // SourceBean query, String type)
    public static SourceBean executeProfilingQuery(
        String queryType,
        String query,
        ArrayList parameters)
        throws Exception {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "QueryExecutor::executeProfilingQuery: invocato");
        DataConnectionManager dataConnectionManager = null;
        DataConnection dataConnection = null;
        SQLCommand sqlCommand = null;
        DataResult dataResult = null;
        ConfigSingleton config = ConfigSingleton.getInstance();
        try {
            dataConnectionManager = DataConnectionManager.getInstance();
            String pool = (String) config.getAttribute("events_pool");
            dataConnection = dataConnectionManager.getConnection(pool);
            String statement =
                (String) config.getAttribute(
                    "EVENTS_STATEMENTS." + query + ".QUERY");
            if (queryType.equalsIgnoreCase(CREATE))
                sqlCommand = dataConnection.createInsertCommand(statement);
            else if (queryType.equalsIgnoreCase(READ))
                sqlCommand = dataConnection.createSelectCommand(statement);
            else if (queryType.equalsIgnoreCase(UPDATE))
                sqlCommand = dataConnection.createUpdateCommand(statement);
            else if (queryType.equalsIgnoreCase(DELETE))
                sqlCommand = dataConnection.createDeleteCommand(statement);
            if (parameters != null) {
                ArrayList inputParameters = new ArrayList();
                Iterator it = parameters.iterator();
                while (it.hasNext())
                    inputParameters.add(
                        dataConnection.createDataField(
                            "",
                            Types.VARCHAR,
                            (String) it.next()));
                dataResult = sqlCommand.execute(inputParameters);
            }
            else
                dataResult = sqlCommand.execute();
            return dataResult.getDataObject().getSourceBean();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "QueryExecutor::executeProfilingQuery:",
                ex);
            throw ex;
        } // catch (Exception ex) try
        finally {
            Utils.releaseResources(dataConnection, sqlCommand, dataResult);
        }
    } // public static DataResult executeQuery(String pool, String queryType,

    // String query, ArrayList inputParameters) throws Exception
    public static SourceBean executeProfilingQuery(
        DataConnection dataConnection,
        String queryType,
        String query,
        ArrayList parameters)
        throws Exception {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "QueryExecutor::executeProfilingQuery: invocato");
        SQLCommand sqlCommand = null;
        DataResult dataResult = null;
        ConfigSingleton config = ConfigSingleton.getInstance();
        try {
            String statement =
                (String) config.getAttribute(
                    "EVENTS_STATEMENTS." + query + ".QUERY");
            if (queryType.equalsIgnoreCase(CREATE))
                sqlCommand = dataConnection.createInsertCommand(statement);
            else if (queryType.equalsIgnoreCase(READ))
                sqlCommand = dataConnection.createSelectCommand(statement);
            else if (queryType.equalsIgnoreCase(UPDATE))
                sqlCommand = dataConnection.createUpdateCommand(statement);
            else if (queryType.equalsIgnoreCase(DELETE))
                sqlCommand = dataConnection.createDeleteCommand(statement);
            if (parameters != null) {
                ArrayList inputParameters = new ArrayList();
                Iterator it = parameters.iterator();
                while (it.hasNext())
                    inputParameters.add(
                        dataConnection.createDataField(
                            "",
                            Types.VARCHAR,
                            (String) it.next()));
                dataResult = sqlCommand.execute(inputParameters);
            }
            else
                dataResult = sqlCommand.execute();
            return dataResult.getDataObject().getSourceBean();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "QueryExecutor::executeProfilingQuery:",
                ex);
            throw ex;
        } // catch (Exception ex) try
        finally {
            Utils.releaseResources(null, sqlCommand, dataResult);
        }
    } // public static SourceBean executeQuery(DataConnection dataConnection,
    // String pool, String queryType, String query, ArrayList inputParameters) throws Exception
} // public abstract class QueryExecutor
