package it.eng.spago.validation;

import java.util.List;

import it.eng.spago.base.CloneableObject;
import it.eng.spago.error.EMFUserError;;

public class EMFValidationError extends EMFUserError {

	// The name of the field that has originated the error
	private String fieldName = null;
	/**
     * Costruisce un oggetto di tipo <code>EMFValidationError</code> identificandolo  tramite  una severity e un
     * codice di errore .
     * @param severity severity dell'errore.
     * @param fieldName il nome del field che ha generato l'errore
     * @param code codice di errore.
     */
    public EMFValidationError(String severity, String fieldName, int code) {
        super(severity, code, null, null);
        this.fieldName = fieldName;
    } // public EMFUserError(String severity, int code)

	/**
     * Costruisce un oggetto di tipo <code>EMFValidationError</code> identificandolo  tramite  una severity e un
     * codice di errore .
     * @param severity severity dell'errore.
     * @param code codice di errore.
     */
    public EMFValidationError(String severity, int code) {
        super(severity, code, null, null);
    } // public EMFUserError(String severity, int code)

    /**
     * Costruisce un oggetto di tipo <code>EMFValidationError</code> identificandolo  tramite  una severity ,un
     * codice di errore e una collezione di parametri che andranno a sostituire i caratteri <em>%</em> nella
     * stringa di descrizione.
     * @param severity severity dell'errore.
     * @param code codice di errore.
     * @parma fieldName il nome del field che ha generato l'errore
     * @param params vettore di parametri che  verranno inseriti nella stringa di descrizione.
     */
    public EMFValidationError(String severity, String fieldName, int code, List params) {
        super(severity, code, params, null);
        this.fieldName = fieldName;
    } // public EMFUserError(String severity, int code, List params)

    
    /**
     * Costruisce un oggetto di tipo <code>EMFValidationError</code> identificandolo  tramite  una severity ,un
     * codice di errore e una collezione di parametri che andranno a sostituire i caratteri <em>%</em> nella
     * stringa di descrizione.
     * @param severity severity dell'errore.
     * @param code codice di errore.
     * @param params vettore di parametri che  verranno inseriti nella stringa di descrizione.
     */
    public EMFValidationError(String severity, int code, List params) {
        super(severity, code, params, null);
    } // public EMFUserError(String severity, int code, List params)

    
    /**
     * Costruisce un oggetto di tipo <code>EMFUserError</code> identificandolo  tramite  una severity ,un
     * codice di errore , una collezione di parametri che andranno a sostituire i caratteri <em>%</em> nella
     * stringa di descrizione e un oggetto di qualsiasi natura.
     * @param severity severity dell'errore.
     * @param code codice di errore.
     * @param params vettore di parametri che  verranno inseriti nella stringa di descrizione.
     * @param additionalInfo oggetto di qualsiasi natura.
     */
    public EMFValidationError(String severity, int code, List params, Object additionalInfo) {
        super(severity, code, params, additionalInfo);
    } // public EMFUserError(String severity, int code, List params, Object additionalInfo)

    /**
     * Costruisce un oggetto di tipo <code>EMFUserError</code> utilizzando lo stato del parametro
     * in input .
     * @param EMFUserError oggetto della stessa classe.
     */
    public EMFValidationError(EMFValidationError valError) {
        super(valError);
        this.fieldName = valError.getFieldName();
    } // public EMFUserError(EMFUserError userError)

	public String getFieldName() {
		return fieldName;
	}

	public void setFieldName(String fieldName) {
		this.fieldName = fieldName;
	}
	/**
     * Ritorna un clone dell'oggetto stesso.
     * @return CloneableObject  il clone dell'oggetto.
     * @see CloneableObject
     */
    public CloneableObject cloneObject() {
        return new EMFValidationError(this);
    } // public CloneableObject cloneObject()

}
