package it.eng.spago.validation;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

public class FieldValidatorFactory {

	public static FieldValidatorIFace getFieldValidator(String fieldType){
		
		TracerSingleton.log(Constants.NOME_MODULO,
				TracerSingleton.INFORMATION, "FieldValidatorFactory:: getFieldValidator:: Start Getting validator for type ["+fieldType+"]");
		
		SourceBean fieldValidatorSourceBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("FIELD-VALIDATORS.FIELD-VALIDATOR", "fieldType", fieldType);
		
		String fieldValidatorImplementingClass = (String)fieldValidatorSourceBean.getAttribute("fieldValidatorClass");
		
		TracerSingleton.log(Constants.NOME_MODULO,
				TracerSingleton.INFORMATION, "FieldValidatorFactory:: getFieldValidator:: fieldValidatorImplementingClass ["+fieldValidatorImplementingClass+"]");
		
		FieldValidatorIFace fieldValidator = null;
		try{
			 fieldValidator = (FieldValidatorIFace)Class.forName(fieldValidatorImplementingClass).newInstance(); 
		}catch (Exception e) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "Exception instantiating validator for type " +fieldType + "" + e);
			return null;
		}
		
		TracerSingleton.log(Constants.NOME_MODULO,
				TracerSingleton.INFORMATION, "FieldValidatorFactory:: getFieldValidator:: fieldValidator Created");
		
		SourceBean config = (SourceBean)fieldValidatorSourceBean.getAttribute("CONFIG"); 
		
		if (config != null){
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.INFORMATION, "FieldValidatorFactory:: getFieldValidator:: fieldValidator Initialize with configuration", config);
			fieldValidator.init(config);
		}
		
		return fieldValidator;
	}
	
}
