/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorHandler;

public interface ValidatorIFace {

	/**
	 * Method check Effettua i controlli formali sui dati della sezione.
	 * @param request Richiesta da controllare.
	 * @param errorHandler Gestore degli errori a cui segnalare le anomalie.
	 * @return boolean true se la verifica deve proseguire con le altre sezioni, false altrimenti
	 */
	public abstract boolean check(final SourceBean request, EMFErrorHandler errorHandler);

	/**
	 * Method validate. Effettua i controlli di dominio sui dati della sezione.
	 * Questo metodo viene invocato solo se i controlli formali non hanno prodotto errori.
	 * @param request Richiesta da controllare.
	 * @param errorHandler Gestore degli errori a cui segnalare le anomalie.
	 * @return boolean. true se la verifica deve proseguire con gli altri validatori, false altrimenti
	 */
	public abstract boolean validate(final SourceBean request, EMFErrorHandler errorHandler);
	
	/**
	 * Method confirm. Effettua i controlli finali che richiedono una conferma da parte dell'utente.
	 * Questo metodo deve generare al pi un messaggio che verr presentato in un alert
	 * con i bottoni OK e Annulla.
	 * Questo metodo viene invocato solo se i controlli formali (check) e i
	 * controlli di dominio (validate) non producono errori.
	 * L'eventuale errore prodotto da questo metodo deve avere severity INFORMATION.
	 * Perch il meccanismo funzioni, nella pagina JSP di dettaglio deve essere preswente il tag
	 * <bk:error/>, inoltre nel form che si chiama "thisform" deve essere presente un campo
	 * hidden definito come segue:
	 * <input type="hidden" name="BK_SKIP_CONFIRM" value="">
	 * Se l'utente preme OK, alla richiesta di conferma, viene rieseguito il submit del
	 * form, pertanto vengono rieseguiti tutti i controlli ma si evita
	 * di invocare il metodo confirm, in modo che l'esecuzione prosegua senza
	 * ulteriori alert.
	 * @param request Richiesta da controllare.
	 * @param errorHandler Gestore degli errori a cui segnalare le anomalie.
	 */
	public abstract void confirm(final SourceBean request, EMFErrorHandler errorHandler);
	
}
