/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.FieldValidatorIFace;

/**
 * @author zoppello
 *
 * An abstract class that implements FieldValidatorIFace, in this class there is the
 * code for handling of the TYPED_SERVICE_REQUEST source bean that will contains the field with
 * alias after the validation
 */
public abstract class AbstractFieldValidator implements FieldValidatorIFace {

	/**
	 * The Key of the element that represent the Typed service Request
	 */
	public static final String TYPED_SERVICE_REQUEST = "TYPED_SERVICE_REQUEST";
	
	/**
	 * The configuration of the field validator, it will contain the contente defined
	 * in <CONFIG/> section of each validator
	 */
	private SourceBean config = null;
	
	/**
	 * Save in the Typed Service Request the attribute with name and value
	 * 
	 * Subclasses must use this method to save the field converted with typed value and alias
	 * 
	 * @param serviceRequest The Spago Service Request 
	 * @param name The name of the attribute to store
	 * @param value The value to store 
	 */
	public void saveTypedValue(SourceBean serviceRequest, String name, Object value){
		try{
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.DEBUG, "AbstractFieldValidator::saveTypedValue: name ["+name + "] value ["+value+"]");
			getTypedServiceRequest(serviceRequest).setAttribute(name, value);
			
		}catch (Exception e) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "AbstractFieldValidator::saveTypedValue:", e);
		}
	}
	
	/**
	 * This method get the TYPED_SERVICE_REQUEST SourceBean from Service Request, if service Request doesn't have
	 * the Typed Service Request this will be created
	 * 
	 * @param serviceRequest The Spago Service Request
	 * @return The Typed Service Request Source Bean
	 * @throws SourceBeanException if some errors occurs istantiating service request source bean
	 */
	private SourceBean getTypedServiceRequest(SourceBean serviceRequest) throws SourceBeanException{
		SourceBean typedServiceRequest  = (SourceBean)serviceRequest.getAttribute(TYPED_SERVICE_REQUEST);
		if ( typedServiceRequest == null){
			typedServiceRequest = new SourceBean(TYPED_SERVICE_REQUEST);
			serviceRequest.setAttribute(typedServiceRequest);
		}
		return typedServiceRequest;
		
	}

	/**
	 * The getter method for the configuration of the field Validator
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#getConfig()
	 */
	public SourceBean getConfig() {
		return this.config;
	}

	/**
	 * Initialize The validator with his configuration 
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#init(it.eng.spago.base.SourceBean)
	 */
	public void init(SourceBean config) {
		this.config = config;	
	}
	
}
