/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
/**
 * @author zoppello
 *
 * This class checks if a field is a valid date.
 * 
 * This implementation need that this validator should be configured with the a String
 * representing the date format in his configuration in the fieldvalidators.xml
 * 
 * For Example:
 * 
 * <FIELD-VALIDATOR fieldType="DATE" fieldValidatorClass="it.eng.spago.validation.fieldvalidators.DateValidator">
 *		<CONFIG>
 *			<DATE-FORMAT dateFormat="dd/MM/yyyy"/>
 *		</CONFIG>
 *	</FIELD-VALIDATOR>
 *
 *	If the config section is not present the format dd/MM/yyyy will be used.
 *
 *  This Validator will convert the original value in a java.util.Date if it succeed and  
 *  this will be putted  in TYPED_SERVICE_REQUEST with an alias if this is configured in validation.xml file
 */
public class DateValidator extends AbstractFieldValidator {

	
	/**
	 * The error code used if the validation fails
	 */
	public static final int ERRORE_CAMPO_DATA = 10105;
	
	/** 
	 * Implements the logic described in JavaDoc of the class
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#validateField(it.eng.spago.base.SourceBean, java.lang.String, java.lang.String, java.util.Map)
	 */
	public void validateField(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams) throws EMFValidationError {
		
		SourceBean config = getConfig();
		
		String formatDate = "dd/MM/yyyy";
		
		if (config != null){
			formatDate = (String)getConfig().getAttribute("DATE-FORMAT.dateFormat");
		}
		
		SimpleDateFormat sdf = new SimpleDateFormat(formatDate);
		
		Date parsedDate = null;
		
		List params = new ArrayList();
		params.add(fieldName);
		
		try{
			
			parsedDate = sdf.parse(value);
			
			String fieldAlias = (String)fieldValidationParams.get(SyntaxChecker.PARAM_FIELD_ALIAS);
			
			if (fieldAlias == null){
				fieldAlias = fieldName;
			}
			
			saveTypedValue(serviceRequest, fieldAlias, parsedDate);
		
		}catch(Exception e){
		
			throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CAMPO_DATA, params);
		
		}
			
	
		
		
	}
}
