/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.validator.GenericValidator;
/**
 * @author zoppello
 *
 * This class checks if a field is a valid according to regular exepression defined in validation.xml file
 * 
 * This Validator doesn't apply type conversion, if it succed the original value will be put in TYPED_SERVICE_REQUEST
 * with an alias if this is configured in validation.xml file
 */
public class RegExpValidator extends AbstractFieldValidator {

	
	public static final int ERRORE_CAMPO_RE = 10110;
	/** 
	 * Implements the logic described in JavaDoc of the class
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#validateField(it.eng.spago.base.SourceBean, java.lang.String, java.lang.String, java.util.Map)
	 */
	public void validateField(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams) throws EMFValidationError {
		 
		String regExp = (String)fieldValidationParams.get(SyntaxChecker.PARAM_REGXEP);
		
		
		if (!GenericValidator.matchRegexp(value, regExp)){
			List errorParams = new ArrayList();
			errorParams.add(fieldName);
			errorParams.add(regExp);
			throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, ERRORE_CAMPO_RE, errorParams);
		}	
		
	}
}
