/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.impl;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.util.StringUtils;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.FieldValidatorFactory;
import it.eng.spago.validation.FieldValidatorIFace;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.validator.GenericValidator;
import org.apache.commons.validator.UrlValidator;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           COntrollo su campo alfa numerico
* 										    
**/

public class SyntaxChecker 
{
	public static final String FIELD_GENERIC = "GENERIC";
	public static final String FIELD_FISCALCODE = "FISCALCODE";
	public static final String FIELD_EMAIL = "EMAIL";
	public static final String FIELD_URL = "URL";
	public static final String FIELD_DATE = "DATE";
	public static final String FIELD_NUMERIC = "NUMERIC";	
	public static final String FIELD_ALFANUMERIC = "ALFANUMERIC";
	public static final String FIELD_LETTERSTRING = "LETTERSTRING";
	public static final String FIELD_REGULAR_EXPRESSION = "REGEXP";
	public static final String FIELD_IMPORTO = "DECIMAL";
	public static final String FIELD_NAME = "NAME";
	
	public static final String FIELD_GENERIC_DEPRECATED = "1";
	public static final String FIELD_FISCALCODE_DEPRECATED = "2";
	public static final String FIELD_EMAIL_DEPRECATED = "3";
	public static final String FIELD_URL_DEPRECATED = "4";
	public static final String FIELD_DATE_DEPRECATED = "5";
	public static final String FIELD_NUMERIC_DEPRECATED = "6";	
	public static final String FIELD_ALFANUMERIC_DEPRECATED = "7";
	public static final String FIELD_LETTERSTRING_DEPRECATED = "8";
	public static final String FIELD_REGULAR_EXPRESSION_DEPRECATED = "10";	
	public static final String FIELD_IMPORTO_DEPRECATED = "11";
	public static final String FIELD_NAME_DEPRECATED = "12";
	
	
	//Codici Di Errore
	public static final int ERRORE_CAMPO_OBBLIGATORIO = 10100;
	public static final int ERRORE_CODICE_CONTROLLO = 10112;
	public static final int ERRORE_MAXLENGTH = 10117;		
	
	public static final String DEFAULT_RANGE = "";
	public static final int DEFAULT_CIFREDECIMALI = -1;
	public static final int DEFAULT_MAXCIFREDECIMALI = -1;
	
	
	//Name of the parameters passed in Map to fieldValidators	
	public static final String PARAM_REGXEP = "PARAM_REGXEP";
	public static final String PARAM_MAX_DECIMALS = "PARAM_MAX_DECIMALS";
	public static final String PARAM_DECIMALS = "PARAM_DECIMALS";
	public static final String PARAM_FIELD_ALIAS = "PARAM_FIELD_ALIAS";

	/**
	 * This function serves to conform previous validation syntax to the new syntax, infact
	 * in previous version field type were expressed by numbers code this function only checks if field type
	 * is written with old syntax an convert it in the string with the STRING code
	 * @param fieldType The field type that arrive to check() method
	 * @return the eventually converted fieldType
	 */
	private static String retroCompatibilityConversion(String fieldType){
		if (fieldType.equalsIgnoreCase(FIELD_FISCALCODE_DEPRECATED)){
			return FIELD_FISCALCODE;
		}else if (fieldType.equalsIgnoreCase(FIELD_EMAIL_DEPRECATED)){
			return FIELD_EMAIL;
		}else if (fieldType.equalsIgnoreCase(FIELD_URL_DEPRECATED)){
				return FIELD_URL;
		}else if (fieldType.equalsIgnoreCase(FIELD_DATE_DEPRECATED)){
			return FIELD_DATE;
		}else if (fieldType.equalsIgnoreCase(FIELD_NUMERIC_DEPRECATED)){	
			return FIELD_NUMERIC;
		}else if (fieldType.equalsIgnoreCase(FIELD_ALFANUMERIC_DEPRECATED)){
			return FIELD_ALFANUMERIC;
		}else if (fieldType.equalsIgnoreCase(FIELD_LETTERSTRING_DEPRECATED)){
			return FIELD_LETTERSTRING;
		}else if (fieldType.equalsIgnoreCase(FIELD_REGULAR_EXPRESSION_DEPRECATED)){
			return FIELD_REGULAR_EXPRESSION;
		}else if (fieldType.equalsIgnoreCase(FIELD_IMPORTO_DEPRECATED)){
			return FIELD_IMPORTO;
		}else if (fieldType.equalsIgnoreCase(FIELD_NAME_DEPRECATED)){
			return FIELD_NAME;
		}else if (fieldType.equalsIgnoreCase(FIELD_GENERIC_DEPRECATED)){
			return FIELD_GENERIC;
		}else{
			return fieldType;
		}
	}
	/**
	 * Method check. Verifica la validit di un campo di input.
	 * @param mandatory true se  il campo  obbligatorio, false altrimenti.
	 * @param tipoCampo tipo del campo, deve essere uno dei valori CAMPO_CODICE_FISCALE..CAMPO_DATA
	 * @param value valore da verificare
	 * @return boolean true se il campo supera i controlli formali, false altrimenti
	 */
	public static boolean check(
		SourceBean serviceRequest,
		final boolean mandatory,
		final int maxlength,
		final String fieldType,
		final String value,
		final String fieldName,
		final String fieldAliasAfterValidation,
		final String regExp,
		final int decimals,
		final int maxDecimals,

		EMFErrorHandler errorHandler) {
		boolean risultato = true;
		
		//Controllo campo obbligatorio
		if (mandatory && GenericValidator.isBlankOrNull(value))
		{
			List params = new ArrayList();
			params.add(fieldName);
			errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CAMPO_OBBLIGATORIO));	
			risultato = false;
		}

		//Controllo maxlength
		if (risultato && maxlength!=-1  
			&& (value!=null && value.length()>maxlength) )
		{
			List params = new ArrayList();
			params.add(fieldName);
			errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_MAXLENGTH));
			risultato = false;
		}

		if (risultato && !GenericValidator.isBlankOrNull(value))	
		{
			
		    	// Converti i tipi deprecati nei tipi normal
		    	String remappedFieldType = retroCompatibilityConversion(fieldType);
		    	
		    	FieldValidatorIFace fieldValidator= (FieldValidatorIFace) FieldValidatorFactory.getFieldValidator(fieldType);
		    	
		    	if (fieldValidator == null){
					List params = new ArrayList();
		    		params.clear();
					params.add(String.valueOf(remappedFieldType));
					errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CODICE_CONTROLLO));	
		    	}
		    	
		    	
		    	
		    	// Eventuali parametri utili per la validazione
		    	Map fieldValidationParams = new HashMap();
		    	fieldValidationParams.put(PARAM_REGXEP,regExp);
		    	fieldValidationParams.put(PARAM_MAX_DECIMALS, new Integer(maxDecimals));
		    	fieldValidationParams.put(PARAM_DECIMALS, new Integer(decimals));
		    	if (fieldAliasAfterValidation != null){
		    		fieldValidationParams.put(PARAM_FIELD_ALIAS, fieldAliasAfterValidation);
		    	}
		    	try{
		    		fieldValidator.validateField(serviceRequest, fieldName, value, fieldValidationParams) ;
		    	}catch(EMFValidationError emfve){
		    		risultato = false;
		    		errorHandler.addError(emfve);
		    	}
		}
		
		return risultato;
	}	

}
