/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.impl;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           The Validation comes a class
* 										    
**/
/**
 * @author boccalon
 *
 */
 
public final class ValidatorLocator {

    private static ValidatorLocator instance = null;
    private HashMap validators = null; // HashMap of Strings (class names)
    
	public final static String VALIDATION_CONFIG_HEADER = "VALIDATIONS.SERVICE";
	public final static String VALIDATOR_CONFIG_HEADER  = "VALIDATORS.VALIDATOR";
	public final static String VALIDATION               = "VALIDATION";
	public final static String VALIDATION_SERVICE_NAME  = "NAME";
	
    /**
     * In questo costruttore tutti i files XML di configurazione vengono letti e con le informazioni in essi 
     * contenute vengono create delle istanze di SourceBean; questo permette di accedere alle informazioni 
     * utilizzando i metodi della classe SourceBean. 
     */
    private ValidatorLocator() throws Exception {
    	validators = new HashMap();
    } // private ValidatorLocator() throws Exception

    public static ValidatorLocator getInstance() {
        if (instance == null) {
            synchronized (ValidatorLocator.class) {
                if (instance == null) {
                    try {
                        instance = new ValidatorLocator();
                    } // try
                    catch (Exception ex) {
                        ex.printStackTrace();
                    } // catch(Exception ex) try
                } // if (_instance == null)
            } // synchronized(ValidatorLocator.class)
        } // if (_instance == null)
        return instance;
    } // public static ValidatorLocator getInstance()
    
	/**
	 * Restituisce la classe di validazione il cui nome logico  fornito come parametro. 
	 * 
	 * @param validatorName Nome logico del validatore.
	 * 
	 * Esempio :
		<VALIDATORS>
		    <VALIDATOR sectionName="ANDATA" class="com.laboratorioalfa.sample.business.validation.AnagraficaValidator"/>
		    <VALIDATOR sectionName="RITORNO" class="com.laboratorioalfa.sample.business.validation.AnagraficaValidator"/>
		</VALIDATORS>
	 */
	
	public Collection getValidators(SourceBean validationStructure) {
		Collection classNames = (Collection)validators.get(validationStructure);
    	if (classNames == null) {

            synchronized (ValidatorLocator.class) {
            	ConfigSingleton config = ConfigSingleton.getInstance();
    			String validatorNamesAttribute = (String)validationStructure.getAttribute("VALIDATORS");
    			if( validatorNamesAttribute != null ) {
	    			// La seguente istruzione  commentata perch  compilabile solo con il JDK 1.4
					//String[] validatorNames = validatorNamesAttribute.split(",");
	    			
	    			// Con il JDK 1.3 bisogna appoggiarsi alla libreria regexp.jar che fornisce l'oggetto RE o
    				// al StringTokenizer
    				StringTokenizer stk = new StringTokenizer(validatorNamesAttribute, ",");

//					classNames = new ArrayList(validatorNames.length);
					classNames = new ArrayList(stk.countTokens());
					
					// Adesso tramite i nomi dei validatori, reperisco i nomi delle corrispondenti classi
//					for (int i = 0; i < validatorNames.length; i++) {
					for (; stk.hasMoreTokens(); ) {
//						String validatorName = validatorNames[i];
						String validatorName = stk.nextToken();
						SourceBean validatorInfo = (SourceBean)config.getFilteredSourceBeanAttribute(VALIDATOR_CONFIG_HEADER, "name", validatorName);
						String className = (String)validatorInfo.getAttribute("class");
						if (className != null) {
							classNames.add(className);
						}
					}
					
		    		// Metto il validatore in cache
		    		validators.put(validationStructure, classNames);
    			}
            }// synchronized (ValidatorLocator.class)
    	}

		Collection result = null;
		
		if (classNames != null) {
			result = new ArrayList();
			for (Iterator iter = classNames.iterator(); iter.hasNext();) {
				String className = (String)iter.next();

				try {
					Class validatorClass = Class.forName(className);
					Constructor validatorConstructor = validatorClass.getConstructor(new Class[0]);
					result.add(validatorConstructor.newInstance(new Object[0]));
				} catch (ClassNotFoundException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (NoSuchMethodException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (InstantiationException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (InvocationTargetException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (IllegalAccessException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				}
			}
		}
    	
    	return result;
    }

}

