/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.ejbchannel;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.transaction.UserTransaction;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.event.EventExecutor;
import it.eng.spago.tracing.TracerSingleton;
import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class BusinessDelegateEJBBean
    implements BusinessDelegateEJBBusiness, SessionBean {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private SessionContext _ejbContext = null;

    public void service(SourceBean serviceRequest, SourceBean serviceResponse)
        throws Exception {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "BusinessDelegateEJBBean::service: serviceRequest",
            serviceRequest);
        Monitor monitor =
            MonitorFactory.start(
                "controller.adapter.ejb.local.business-delegate");
        RequestContainer requestContainer = new RequestContainer();
        requestContainer.setSessionContainer(new SessionContainer(true));
        requestContainer.setChannelType(Constants.EJB_CHANNEL);
        requestContainer.setServiceRequest(serviceRequest);
        ResponseContainer responseContainer = new ResponseContainer();
        EMFErrorHandler emfErrorHandler = new EMFErrorHandler();
        responseContainer.setErrorHandler(emfErrorHandler);
        responseContainer.setServiceResponse(serviceResponse);
        RequestContextIFace requestContext =
            new DefaultRequestContext(requestContainer, responseContainer);
        requestContainer.setRequestDistributed(true);
        CoordinatorIFace coordinator =
            DispatcherManager.getCoordinator(requestContext);
        if (coordinator == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "BusinessDelegateEJBBean::service: coordinator nullo !");
            emfErrorHandler.addError(
                new EMFInternalError(
                    EMFErrorSeverity.ERROR,
                    "Coordinatore non trovato !"));
        } // if (coordinator == null)
        else {
            ((RequestContextIFace) coordinator).setRequestContext(
                requestContext);
            UserTransaction userTx = _ejbContext.getUserTransaction();
            RequestContainer.setRequestContainer(requestContainer);
            try {
                userTx.begin();
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "BusinessDelegateEJBBean::service: begin transaction");
                coordinator.service(
                    requestContainer.getServiceRequest(),
                    responseContainer.getServiceResponse());
                if (emfErrorHandler
                    .isOKBySeverity(EMFErrorSeverity.BLOCKING)) {
                    userTx.commit();
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.DEBUG,
                        "BusinessDelegateEJBBean::service: commit transaction");
                } // if (emfErrorHandler.isOKBySeverity(EMFErrorSeverity.BLOCKING))
                else {
                    userTx.rollback();
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "BusinessDelegateEJBBean::service: errore BLOCKING nel servizio, rolling back transaction !");
                } // if (emfErrorHandler.isOKBySeverity(EMFErrorSeverity.BLOCKING)) else
            } // try
            catch (Exception ex) {
                userTx.rollback();
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "BusinessDelegateEJBBean::service: errore nel servizio, rolling back transaction !");
                emfErrorHandler.addError(
                    new EMFInternalError(EMFErrorSeverity.ERROR, ex));
            } // catch (Excpetion ex) try
            RequestContainer.delRequestContainer();
            ((RequestContextIFace) coordinator).setRequestContext(null);
        } // if (coordinator == null) else
        requestContainer.setRequestDistributed(false);
        try {
            EventExecutor.service(requestContainer, responseContainer);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "BusinessDelegateEJBBean::service:",
                ex);
            emfErrorHandler.addError(
                new EMFInternalError(EMFErrorSeverity.ERROR, ex));
        } // catch (Exception ex)
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "BusinessDelegateEJBBean::service: serviceResponse",
            serviceResponse);
        monitor.stop();
    } // public void service(SourceBean serviceRequest, SourceBean

    // serviceResponse) throws Exception
    public SessionContext getSessionContext() {
        return _ejbContext;
    } // public SessionContext getSessionContext()

    public void setSessionContext(SessionContext ctx) {
        _ejbContext = ctx;
    } // public void setSessionContext(SessionContext ctx)

    public void ejbCreate() throws CreateException {
    } // public void ejbCreate() throws CreateException

    public void ejbActivate() {
    } // public void ejbActivate()

    public void ejbPassivate() {
    } // public void ejbPassivate()

    public void ejbRemove() {
    } // public void ejbRemove()
} // public class BusinessDelegateEJBBean implements BusinessDelegateEJBBusiness, SessionBean
