/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.ejbchannel;

import java.io.Serializable;

import javax.naming.Context;
import javax.naming.InitialContext;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

public class BusinessDelegateEJBLocalProxy implements Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String _jndi = null;

    public BusinessDelegateEJBLocalProxy(String jndi) {
        _jndi = jndi;
        if (_jndi == null) {
            ConfigSingleton configure = ConfigSingleton.getInstance();
            SourceBean proxy =
                (SourceBean) configure.getFilteredSourceBeanAttribute(
                    "PROXIES.PROXY",
                    "NAME",
                    "BUSINESS_DELEGATE_EJB_LOCAL_PROXY");
            if (proxy != null)
                _jndi = (String) proxy.getAttribute("jndi");
        } // if (_jndi == null)
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "BusinessDelegateEJBLocalProxy::BusinessDelegateEJBLocalProxy: jndi ["
                + _jndi
                + "]");
    } // public BusinessDelegateEJBLocalProxy(String jndi)

    public void service(SourceBean serviceRequest, SourceBean serviceResponse)
        throws Exception {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "BusinessDelegateEJBLocalProxy::service: invocato");
        BusinessDelegateEJBLocal businessDelegateEJBLocal = null;
        try {
            Context ctx = new InitialContext();
            BusinessDelegateEJBLocalHome businessDelegateEJBLocalHome =
                (BusinessDelegateEJBLocalHome) ctx.lookup(_jndi);
            businessDelegateEJBLocal = businessDelegateEJBLocalHome.create();
            businessDelegateEJBLocal.service(serviceRequest, serviceResponse);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "BusinessDelegateEJBLocalProxy::service:",
                ex);
        } // catch (CreateException ex)
        finally {
            if (businessDelegateEJBLocal != null)
                businessDelegateEJBLocal.remove();
        } // finally
    } // public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception
} // public class BusinessDelegateEJBLocalProxy implements Serializable
