/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.ejbchannel;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.transaction.UserTransaction;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.tracing.TracerSingleton;
import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class SessionFacadeEJBBean
    implements SessionFacadeEJBBusiness, SessionBean {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private SessionContext _ejbContext = null;

    public void service(RequestContextIFace requestContext) throws Exception {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "SessionFacadeEJBBean::service: invocato");
        Monitor monitor =
            MonitorFactory.start("controller.adapter.ejb.local.session-facade");
        RequestContainer requestContainer =
            requestContext.getRequestContainer();
        ResponseContainer responseContainer =
            requestContext.getResponseContainer();
        CoordinatorIFace coordinator = null;
        UserTransaction userTx = null;
        try {
            requestContainer.setRequestDistributed(true);
            coordinator = DispatcherManager.getCoordinator(requestContext);
            ((RequestContextIFace) coordinator).setRequestContext(
                requestContext);
            userTx = _ejbContext.getUserTransaction();
            userTx.begin();
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "SessionFacadeEJBBean::service: begin transaction");
            coordinator.service(
                requestContainer.getServiceRequest(),
                responseContainer.getServiceResponse());
            if (requestContext
                .getErrorHandler()
                .isOKBySeverity(EMFErrorSeverity.BLOCKING)) {
                userTx.commit();
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "SessionFacadeEJBBean::service: commit transaction");
            } // if (requestContext.getErrorHandler().isOKBySeverity(EMFErrorSeverity.BLOCKING))
            else {
                userTx.rollback();
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "SessionFacadeEJBBean::service: errore BLOCKING nel servizio, rolling back transaction !");
            } // if (requestContext.getErrorHandler().isOKBySeverity(EMFErrorSeverity.BLOCKING)) else
        } // try
        catch (Exception ex) {
            if (userTx != null) {
                userTx.rollback();
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "SessionFacadeEJBBean::service: errore nel servizio, rolling back transaction !");
            } // if (userTx != null)
            throw ex;
        } // catch (Excpetion ex) try
        finally {
            ((RequestContextIFace) coordinator).setRequestContext(null);
            requestContainer.setRequestDistributed(false);
            monitor.stop();
        } // finally
    } // public void service(RequestContextIFace requestContext) throws Exception

    public SessionContext getSessionContext() {
        return _ejbContext;
    } // public SessionContext getSessionContext()

    public void setSessionContext(SessionContext ctx) {
        _ejbContext = ctx;
    } // public void setSessionContext(SessionContext ctx)

    public void ejbCreate() throws CreateException {
    } // public void ejbCreate() throws CreateException

    public void ejbActivate() {
    } // public void ejbActivate()

    public void ejbPassivate() {
    } // public void ejbPassivate()

    public void ejbRemove() {
    } // public void ejbRemove()
} // public class SessionFacadeEJBBean implements SessionFacadeEJBBusiness, SessionBean
