/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.action;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.DefaultHttpRequestContext;
import it.eng.spago.init.InitializerIFace;


/**
 * La classe <code>AbstractHttpAction</code> &egrave; la superclasse di quelle actions che
 * necessitano di operare esclusivamente nel canale HTTP e hanno necessit? di accedere
 * direttamente agli oggetti HttpServletRequest, HttpServletResponse,ServletConfig
 * (i.e. per ricevere un file in upload).<p>
 * Questa classe mette a disposizione i metodi per recuperare le instanze delle seguenti classi :
 *  <blockquote><pre>
 *  <code>RequestContainer</code> :Il contenitore di oggetti legati ai parametri della richiesta di un servizio.
 *  <code>ResponseContainer</code> :Il contenitore di oggetti legati alla riposta ad una richiesta di un servizio.
 *  <code>HttpServletRequest</code> :L'oggetto che contiene la request gestita dal servlet container
 *  <code>HttpServletResponse</code> :L'oggetto che contiene la response gestita dal servlet container
 *  <em>EMFErrorHandler</code> :Il gestore degli errori.
 *  </blockquote></pre>
 * Nella configurazione XML dell'action e' possibile scrivere uno stream XML(Config) che verr?
 * passato all'istanza dell'action dopo la sua creazione.
 *
 * @see it.eng.spago.base.RequestContainer
 * @see it.eng.spago.base.ResponseContainer
 * @see it.eng.spago.error.EMFErrorHandler
 */
public abstract class AbstractHttpAction extends DefaultHttpRequestContext implements InitializerIFace, ActionIFace {
    private SourceBean _config = null;
    private String _action = null;

    public AbstractHttpAction() {
        super();
        _config = null;
        _action = null;
    } // public AbstractHttpAction()

    public void init(SourceBean config) {
        _config = config;
    } // public void init(SourceBean config)

    public SourceBean getConfig() {
        return _config;
    } // public SourceBean getConfig()

    public String getActionName() {
        return _action;
    } // public String getAction()

    public void setActionName(String action) {
        _action = action;
    } // public void setAction(String action)
} // public abstract class AbstractHttpAction extends
// DefaultHttpRequestContext implements InitializerIFace, ActionIFace
