/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.httpchannel.upload;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

public class UploadFactory {
    private UploadFactory() {
        super();
    } // private UploadFactory()

    public static IUploadHandler getHandler(final String handlerName) throws UploadException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "UploadFactory::getHandler: handlerName [" + handlerName + "]");
        ConfigSingleton configure = ConfigSingleton.getInstance();
        SourceBean uploadBean = (SourceBean)configure.getFilteredSourceBeanAttribute("UPLOAD.UPLOAD_METHODS.UPLOAD_METHOD", "NAME", handlerName);
        if (uploadBean == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "UploadFactory::getHandler: nome dell'handler non valido !");
            throw new UploadException("nome dell'handler [" + handlerName + "] non valido");
        } // if (uploadBean == null)
        String uploadClass = (String)uploadBean.getAttribute("CLASS");
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "UploadFactory::getHandler: uploadClass [" + uploadClass + "]");
        SourceBean uploadConfig = (SourceBean)uploadBean.getAttribute("CONFIG");
        if (uploadConfig == null)
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                "UploadFactory::getHandler: config dell'handler non censito");
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "UploadFactory::getHandler: uploadConfig", uploadConfig);
        IUploadHandler uploadHandler = null;
        try {
            uploadHandler = (IUploadHandler)Class.forName(uploadClass).newInstance();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "UploadFactory::getHandler: classe del uploadHandler non valida", ex);
            throw new UploadException("Classe del uploadHandler [" + handlerName + "] non valida");
        } // catch (Exception ex) try
        
        if (uploadHandler instanceof InitializerIFace) {
            ((InitializerIFace)uploadHandler).init(uploadConfig);
        }
        return uploadHandler;
    } // public static IUploadHandler getHandler(String handlerName) throws UploadException
} // public class UploadFactory
