/**

Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;
/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 28-11-2005	  Zoppello	  Not cause an error if request container or response container are null, but simple trace in log  
* 							  Now it is possible to specify a scope in which this tag search first the value
* 							  The scope attribute of the tag is optional
* 
*  							  scope == SERVICE_REQUEST ( Search the value First In Service Request, and then in service Response )
* 							  scope == SERVICE_RESPONSE ( Search the value First In Service Response, and then in Service Request )		    
**/
/**
 * @author Butano
 * 
 */
public class ReloadFieldTag extends TagSupport {
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String _id = null;
    
    private String _scope = "SERVICE_REQUEST";

	/**
	 * Constructor for GenerateTokenTag.
	 */
	public ReloadFieldTag() {
		super();
	}

    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "ReloadFieldTag::doStartTag:: invocato");
        HttpServletRequest httpRequest = (HttpServletRequest)pageContext.getRequest();
        RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
        ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
      
        if (requestContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ReloadFieldTag::doStartTag:: requestContainer nullo");
        } // if (_content == null)
        
        if (responseContainer == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ReloadFieldTag::doStartTag:: responseContainer nullo");
        } // if (_content == null)
     
        //Default search the field Value in serviceRequest
        String value = "";
        if ((_scope == null) || (_scope.equalsIgnoreCase("SERVICE_REQUEST"))){
        	value  = ((requestContainer != null) 
        			 ? (String)requestContainer.getServiceRequest().getAttribute(_id)
        			 : null);
        	if (value == null){
        		// Value is not in service Request try in service repsponse
        		value = ( (responseContainer != null)
        				? (String)responseContainer.getServiceResponse().getAttribute(_id)
        				: null);
        		if (value == null)
        			value = "";
        	}
        }else if (_scope.equalsIgnoreCase("SERVICE_RESPONSE")){
        	value = ( (responseContainer != null)
       				? (String)responseContainer.getServiceResponse().getAttribute(_id)
       				: null);
        	
        	if (value == null){
       		
        		value  = ((requestContainer != null) 
              			 ? (String)requestContainer.getServiceRequest().getAttribute(_id)
              			 : null);
        	}
       		if (value == null)
       			value = "";
        }
        
        
        if (value == null)
       			value = "";
        	
        StringBuffer _htmlStream = new StringBuffer();        
        _htmlStream.append(value);
        
        try {
            pageContext.getOut().print(_htmlStream);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "ReloadFieldTag::doStartTag:: Impossibile inviare lo stream");
            throw new JspException("Impossibile inviare lo stream");
        } // catch (Exception ex)
        return SKIP_BODY;
    } // public int doStartTag() throws JspException

    public void setId(String id) {
    	_id = id;
    }
    
    public void setScope(String scope) {
    	_scope = scope;
    }
    
    /**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "ReloadFieldTag::doEndTag:: invocato");
        return super.doEndTag();
    } // public int doEndTag() throws JspException
    
    
    

}
